// IcctSign.cpp : Definiert den Einsprungpunkt fr die Konsolenanwendung.
//

#include "idptypes.h"

#include "IcctSign.h"

#include <stdio.h>
#include <memory.h>

#include "sha.h"
#include "rsa.h"
#include "rand.h"
#include "err.h"


// check SSLEAY version
#define SSL090
#ifdef SSL090
	// 0.9.0
	#define RSA_private_encrypt_S(a, b, c, d) RSA_private_encrypt(a, b, c, d, RSA_PKCS1_PADDING)
	#define RSA_public_decrypt_S(a, b, c, d) RSA_public_decrypt(a, b, c, d, RSA_PKCS1_PADDING)
	#define RSA_public_encrypt_S(a, b, c, d) RSA_public_encrypt(a, b, c, d, RSA_PKCS1_PADDING)
	#define RSA_private_decrypt_S(a, b, c, d) RSA_private_decrypt(a, b, c, d, RSA_PKCS1_PADDING)
	#define RSA_generate_key_S(a, b, c) RSA_generate_key(a, b, c, NULL)
#else
	// 0.6.6
	#define RSA_private_encrypt_S RSA_private_encrypt
	#define RSA_public_decrypt_S RSA_public_decrypt
	#define RSA_public_encrypt_S RSA_public_encrypt
	#define RSA_private_decrypt_S RSA_private_decrypt
	#define RSA_generate_key_S RSA_generate_key
#endif

HRESULT ICCT_Signature(UCHAR* pucPrivateKey, int cbPrivateKey, UCHAR* pMessage,
					   ULONG cbMessage, /*[out]*/ UCHAR* pDigest, ULONG* pcbDigest)
{
	RSA *pPrivateKey = RSA_new();
	UCHAR Hash[SHA_DIGEST_LENGTH];

#ifdef UNIX
	if(!d2i_RSAPrivateKey(&pPrivateKey, (const UCHAR**)&pucPrivateKey, cbPrivateKey))
#else
	if(!d2i_RSAPrivateKey(&pPrivateKey, &pucPrivateKey, cbPrivateKey))
#endif //UNIX

	{
		goto err;
	}

	memcpy(Hash, SHA1(pMessage, cbMessage, NULL), SHA_DIGEST_LENGTH);

	if((*pcbDigest =
		RSA_private_encrypt_S(SHA_DIGEST_LENGTH, Hash, pDigest, pPrivateKey)) == -1)
	{
		goto err;
	}

	RSA_free(pPrivateKey);
	return 0;

err:
	RSA_free(pPrivateKey);
	return -1;
}

HRESULT ICCT_Verify(UCHAR* pucPublicKey, int cbPublicKey, UCHAR* pMessage, ULONG cbMessage,
					/*[in]*/ UCHAR* pDigest, ULONG cbDigest)
{
	RSA *pPublicKey = RSA_new();
	UCHAR Hash[20], DecryptedHash[ICCT_KEYLENGTH / 4];

#ifdef UNIX
	if(!d2i_RSAPublicKey(&pPublicKey, (const UCHAR**)&pucPublicKey, cbPublicKey))
#else
	if(!d2i_RSAPublicKey(&pPublicKey, &pucPublicKey, cbPublicKey))
#endif //UNIX
	{
		goto err;
	}

	memcpy(Hash, SHA1(pMessage, cbMessage, NULL), SHA_DIGEST_LENGTH);

	if(RSA_public_decrypt_S(cbDigest, pDigest, DecryptedHash, pPublicKey)
		!= SHA_DIGEST_LENGTH)
	{
		goto err;
	}

	if(memcmp(Hash, DecryptedHash, 20) != 0)
	{
		goto err;
	}

	RSA_free(pPublicKey);
	return 0;

err:
	RSA_free(pPublicKey);
	return -1;
}

HRESULT ICCT_CryptDigest(UCHAR* pucPublicKey, int cbPublicKey,
					UCHAR* pDigest, ULONG cbDigest, UCHAR * pOut, ULONG * cbOut)
{
	RSA *pPublicKey = RSA_new();

#ifdef UNIX
	if(!d2i_RSAPublicKey(&pPublicKey, (const UCHAR**)&pucPublicKey, cbPublicKey))
#else
	if(!d2i_RSAPublicKey(&pPublicKey, &pucPublicKey, cbPublicKey))
#endif //UNIX
	{
		goto err;
	}

	//memcpy(Hash, SHA1(pMessage, cbMessage, NULL), SHA_DIGEST_LENGTH);

	UCHAR pCrypted[1024];
	int cbCrypted;
	if((cbCrypted = RSA_public_encrypt_S(cbDigest, pDigest, pCrypted, pPublicKey))
		== -1)
	{
		goto err;
	}

	if((ULONG) cbCrypted > * cbOut)
	{
		goto err;
	}

	memcpy(pOut, pCrypted, cbCrypted);
	* cbOut = cbCrypted;
	RSA_free(pPublicKey);
	return 0;

err:
	RSA_free(pPublicKey);
	return -1;
}

HRESULT ICCT_DecryptDigest(UCHAR* pucPrivateKey, int cbPrivateKey,
					UCHAR * pIn, ULONG cbIn, UCHAR* pDigest, ULONG * cbDigest)
{
	RSA *pPrivateKey = RSA_new();

#ifdef UNIX
	if(!d2i_RSAPrivateKey(&pPrivateKey, (const UCHAR**)&pucPrivateKey, cbPrivateKey))
#else
	if(!d2i_RSAPrivateKey(&pPrivateKey, &pucPrivateKey, cbPrivateKey))
#endif //UNIX
	{
		goto err;
	}

	if((* cbDigest = RSA_private_decrypt_S(cbIn, pIn, pDigest, pPrivateKey))
		== -1)
	{
		goto err;
	}

	RSA_free(pPrivateKey);
	return 0;

err:
	RSA_free(pPrivateKey);
	return -1;
}

////////////////////////////////////////////////////////////////////////////////////////
// ICCT_GenerateKey():
// memory allocation by caller, 

HRESULT ICCT_GenerateKeys(UCHAR* pucPublicKey, int* cbPublicKey, UCHAR* pucPrivateKey,
						  int* cbPrivateKey, UCHAR* Seed,
						  int cbSeed)
{
	
#ifdef UNIX
	RSA *pPrivateKey = RSA_new_method(NULL);
#else
	RSA *pPrivateKey = RSA_new_method(RSA_PKCS1_SSLeay());
#endif


	UCHAR* pPubK = pucPublicKey;
	UCHAR* pPrivK = pucPrivateKey;
	if(cbSeed) RAND_seed(Seed, cbSeed);

	pPrivateKey = RSA_generate_key_S(ICCT_KEYLENGTH, 65537L, NULL);
	if(!pPrivateKey)
	{
		goto err;
	}

	if((*cbPublicKey = i2d_RSAPublicKey(pPrivateKey, &pPubK)) == -1)
	{
		goto err;
	}

	if((*cbPrivateKey = i2d_RSAPrivateKey(pPrivateKey, &pPrivK)) == -1)
	{
		goto err;
	}


	* cbPrivateKey += RSA_padding_add_none(pPrivK, 1, NULL, 0);

	RSA_free(pPrivateKey);
	RAND_cleanup();
	return 0;

err:
	RSA_free(pPrivateKey);
	RAND_cleanup();

	return -1;
}

/*
void main(int argc, char* argv[])
{
	CHAR* pBuf = "1234567890123456789012345678901234567890";

	UCHAR Digest[ICCT_KEYLENGTH / 8];
	UCHAR pucPublicKey[ICCT_KEYLENGTH / 4];
	UCHAR pucPrivateKey[ICCT_KEYLENGTH];
	ULONG cbDigest;int cbPublicKey, cbPrivateKey;

	if(ICCT_GenerateKeys(pucPublicKey, &cbPublicKey, pucPrivateKey, &cbPrivateKey)
		!= ICCT_S_OK)
	{
		printf("ICCT_GenerateKeys failed\n");
		return;
	}

	if(ICCT_Signature(pucPrivateKey, cbPrivateKey, (UCHAR*)pBuf, strlen(pBuf),
		Digest, &cbDigest) == ICCT_S_OK)
	{
		Dump(Digest, cbDigest);
	}
	else
	{
		printf("ICCT_Signature failed\n");
	}

	if(ICCT_Verify(pucPublicKey, cbPublicKey, (UCHAR*)pBuf, strlen(pBuf),
		Digest, cbDigest) != ICCT_S_OK)
	{
		Dump(Digest, cbDigest);
	}
	else
	{
		printf("ICCT_Verify succeeded!\n");
	}
}
*/
