// cConfiguraionSetup.cpp: C-interface for the Saferpay merchant application

#include <stdio.h>

#include "idpapi.h"
#include "idpctx.h"
#include "idpurl.h"
#include "idputil.h"
#include "cppConfigurationSetup.h"


void IDPCIMPL_ConfigurationSetup::SetConfiguration(const STRING & Path)
{
	UDEBUG("ConfigurationSetup::SetConfiguration()\n");
	m_ctx.Open(Path);
}

void IDPCIMPL_ConfigurationSetup::Reconfigure(const STRING & ServerUrl, const STRING & UserName,
									 const STRING & Password)
{
	UDEBUG("ConfigurationSetup::Reconfigure()\n");
	if(ServerUrl.size())
	{
		m_ctx.SetOption(_T("VTAUTOURL"), ServerUrl);
	}
	m_ctx.SetOption(_T("UserName"), UserName);
	m_ctx.SetOption(_T("Password"), Password);
	m_ctx.Reconfigure();
}

void IDPCIMPL_ConfigurationSetup::SetOption(const STRING & Name, const STRING & Value)
{
	UDEBUG("ConfigurationSetup::SetOption()\n");
	m_ctx.SetOption(Name, Value);
}


void IDPCIMPL_ConfigurationSetup::GetAttribute(const STRING & File, const STRING & Name, STRING & pVal)
{
	UDEBUG("ConfigurationSetup::GetAttribute()\n");
	if(!m_ctx.QueryValue(File, Name, pVal))
		IDPTHROW1(IDP_E_NOT_FOUND);
}

void IDPCIMPL_ConfigurationSetup::SetAttribute(const STRING & File, const STRING & Name,
									  const STRING & newVal)
{
	UDEBUG("ConfigurationSetup::SetAttribute()\n");
	if(!newVal.size()) return;

	m_ctx.SetValue(File, Name, newVal);
}

// C-interface functions
extern "C" {

// factory
IDPEXPORT void DestroyConfigurationSetup(IDP_HANDLE hConfigurationSetup)
{
	delete (IDPCIMPL_ConfigurationSetup*)hConfigurationSetup;
}

IDPEXPORT IDP_HANDLE CreateConfigurationSetup()
{
	IDPCIMPL_ConfigurationSetup *hConfigurationSetup = new IDPCIMPL_ConfigurationSetup;
	return (IDP_HANDLE)hConfigurationSetup;
}


IDPEXPORT HRESULT ConfigurationSetup_SetConfiguration(IDP_HANDLE hConfigurationSetup,
													  const char *pszPath)
{
	IDP_ASSERT_VALID(hConfigurationSetup, IDPCIMPL_ConfigurationSetup)
	try
	{
		((IDPCIMPL_ConfigurationSetup*)hConfigurationSetup)->SetConfiguration(mbstoSTRING(pszPath));
		return 0;
	}
	catch(IDP_Exception e)
	{
		return e.hr;
	}
}

IDPEXPORT HRESULT ConfigurationSetup_Reconfigure(IDP_HANDLE hConfigurationSetup,
												 const char *pszServerURL,
												 const char *pszUserName,
												 const char *pszPassword)
{
	IDP_ASSERT_VALID(hConfigurationSetup, IDPCIMPL_ConfigurationSetup)
	try
	{
		((IDPCIMPL_ConfigurationSetup*)hConfigurationSetup)->Reconfigure(mbstoSTRING(pszServerURL),
			mbstoSTRING(pszUserName), mbstoSTRING(pszPassword));
		return 0;
	}
	catch(IDP_Exception e)
	{
		return e.hr;
	}
}

IDPEXPORT HRESULT ConfigurationSetup_SetOption(IDP_HANDLE hConfigurationSetup,
											   const char *pszName,
											   const char *pszValue)
{
	IDP_ASSERT_VALID(hConfigurationSetup, IDPCIMPL_ConfigurationSetup)
	try
	{
		((IDPCIMPL_ConfigurationSetup*)hConfigurationSetup)->SetOption(mbstoSTRING(pszName),
			mbstoSTRING(pszValue));
		return 0;
	}
	catch(IDP_Exception e)
	{
		return e.hr;
	}
}

IDPEXPORT HRESULT ConfigurationSetup_GetAttribute(IDP_HANDLE hConfigurationSetup,
												  const char *pszFile,
												  const char *pszName, long *pcbSize,
												  char *pValue)
{
	IDP_ASSERT_VALID(hConfigurationSetup, IDPCIMPL_ConfigurationSetup)
	STRING sValue(_T(""));

	try
	{
		((IDPCIMPL_ConfigurationSetup*)hConfigurationSetup)->GetAttribute(mbstoSTRING(pszFile),
			mbstoSTRING(pszName), sValue);
	}
	catch(IDP_Exception e)
	{
		return e.hr;
	}

	char *buf = new char[sValue.size()*2+2];	// MBCS
	HRESULT hr = 0;
#ifdef UNICODE
	sprintf(buf, "%S", sValue.c_str());
#else
	sprintf(buf, "%s", sValue.c_str());
#endif
	long size = strlen(buf)+1;
	if(pcbSize)
	{
		if(*pcbSize >= size)
		{
			if(pValue) strcpy(pValue, buf);
		}
		else hr = IDP_E_BUFFER_SIZE;
		*pcbSize = size;
	}
	else hr = IDP_E_BUFFER_SIZE;

	delete buf;
	return hr;
}

IDPEXPORT HRESULT ConfigurationSetup_SetAttribute(IDP_HANDLE hConfigurationSetup,
												  const char *pszFile,
												  const char *pszName,
												  const char *pszNewVal)
{
	IDP_ASSERT_VALID(hConfigurationSetup, IDPCIMPL_ConfigurationSetup)
	try
	{
		((IDPCIMPL_ConfigurationSetup*)hConfigurationSetup)->SetAttribute(mbstoSTRING(pszFile),
			mbstoSTRING(pszName), mbstoSTRING(pszNewVal));
		return 0;
	}
	catch(IDP_Exception e)
	{
		return e.hr;
	}
}

}	// extern "C"
