// cMessageFactory.cpp: C-interface for the Saferpay merchant application


#include <stdio.h>

#include "idpapi.h"
#include "idputil.h"
#include "cppMessageObject.h"
#include "cppMessageFactory.h"
#include "idpdattr.h"


IDP_HANDLE IDPCIMPL_MessageFactory::CreatePayInit()
{
	UDEBUG("MessageFactory::CreatePayInit()\n");
	IDP_Context ctx;
	ctx.Open(m_Path);

	IDPCIMPL_MessageObject *Msg = (IDPCIMPL_MessageObject*)CreateMessageObject();

	try
	{
		Msg->m_ctx.Open(m_Path);
		Msg->m_msg.Create(&Msg->m_ctx, _T("PayInit"));
		Msg->m_msg.SetAttribute(_T("TOKEN"), ctx.CreateUuid());

		Msg->m_Outgoing = 1;

		// set defaults
		STRING s;
		if(ctx.QueryValue(_T("default.xml"), _T(""), s))
		{
			xmlElementPtr pElem;
			ctx.XmlFromString(s, pElem.Ptr());

			DefaultAttributeName * Names = AttributeNames;
			while(Names->Name)
			{
				STRING tmp;
				if(pElem->getAttribute(Names->Name, tmp))
					Msg->m_msg.SetAttribute(Names->Name, tmp);

				Names++;
			}
		}

		STRING PayInitTTL;
		if(!ctx.QueryValue(_T("default.xml"), _T("PAYINITTTL"), PayInitTTL))
		{
			PayInitTTL = _T("86400");
		}

		Msg->m_msg.SetAttribute(_T("EXPIRATION"), GetTimeDate(tcstod(PayInitTTL.c_str(), 0)));

		return (IDP_HANDLE) Msg;
	}
	catch(IDP_Exception e)
	{
		DestroyMessageObject((IDP_HANDLE)Msg);
		throw e;
		return (IDP_HANDLE)NULL;
	}
}

IDP_HANDLE IDPCIMPL_MessageFactory::VerifyPayConfirm(const STRING & Data, const STRING & Signature)
{
	UDEBUG("MessageFactory::VerifyPayConfirm()\n");
	IDP_Context ctx;

	IDPCIMPL_MessageObject *Msg = (IDPCIMPL_MessageObject*) CreateMessageObject();

	try
	{
		Msg->m_ctx.Open(m_Path);
		Msg->m_msg.Open(&Msg->m_ctx, Data, Signature);
		return (IDP_HANDLE) Msg;
	}
	catch(IDP_Exception e)
	{
		DestroyMessageObject((IDP_HANDLE)Msg);
		throw e;
		return (IDP_HANDLE)NULL;
	}
}

IDP_HANDLE IDPCIMPL_MessageFactory::CreatePayComplete(const STRING & Id, const STRING & Token)
{
	UDEBUG("MessageFactory::CreatePayComplete()\n");

	IDPCIMPL_MessageObject *Msg = (IDPCIMPL_MessageObject*) CreateMessageObject();

	try
	{
		Msg->m_ctx.Open(m_Path);
		Msg->m_msg.Create(&Msg->m_ctx, STRING(_T("PayComplete")));
		Msg->m_Outgoing = 1;

		Msg->m_msg.SetAttribute(STRING(_T("ID")), Id);
		Msg->m_msg.SetAttribute(STRING(_T("TOKEN")), Token);
		return (IDP_HANDLE) Msg;
	}
	catch(IDP_Exception e)
	{
		DestroyMessageObject((IDP_HANDLE)Msg);
		throw e;
		return (IDP_HANDLE)NULL;
	}
}

void IDPCIMPL_MessageFactory::Open(const STRING & Path)
{
	UDEBUG("MessageFactory::Open()\n");
	m_Path = Path;
}

IDP_HANDLE IDPCIMPL_MessageFactory::CreateRequest(const STRING & MsgType)
{
	UDEBUG("MessageFactory::CreateRequest()\n");

	if(MsgType.empty())
		IDPTHROW1(IDP_E_PARAMETER);

	IDPCIMPL_MessageObject *Msg = (IDPCIMPL_MessageObject*) CreateMessageObject();
	IDP_Context ctx;

	try
	{
		Msg->m_ctx.Open(m_Path);
		ctx.Open(m_Path);
		Msg->m_msg.Create(&Msg->m_ctx, MsgType);
		Msg->m_Outgoing = 1;
		return (IDP_HANDLE) Msg;
	}
	catch(IDP_Exception e)
	{
		DestroyMessageObject((IDP_HANDLE)Msg);
		throw e;
		return (IDP_HANDLE)NULL;
	}
}


// C-interface functions
extern "C" {

// factory
IDPEXPORT void DestroyMessageFactory(IDP_HANDLE hMessageFactory)
{
	delete (IDPCIMPL_MessageFactory*)hMessageFactory;
}

IDPEXPORT IDP_HANDLE CreateMessageFactory()
{
	IDPCIMPL_MessageFactory *hMessageFactory = new IDPCIMPL_MessageFactory;
	return (IDP_HANDLE)hMessageFactory;
}


IDPEXPORT HRESULT MessageFactory_VerifyPayConfirm(IDP_HANDLE hMessageFactory,
												  const char *pszData,
												  const char *pszSignature,
												  IDP_HANDLE *phMessageObject)
{
	IDP_ASSERT_VALID(hMessageFactory, IDPCIMPL_MessageFactory)
	try
	{
		*phMessageObject = ((IDPCIMPL_MessageFactory*)hMessageFactory)->VerifyPayConfirm(
			mbstoSTRING(pszData), mbstoSTRING(pszSignature));
		return 0;
	}
	catch(IDP_Exception e)
	{
		*phMessageObject = (IDP_HANDLE)0;
		return e.hr;
	}
}

IDPEXPORT HRESULT MessageFactory_CreatePayInit(IDP_HANDLE hMessageFactory,
											   IDP_HANDLE *phMessageObject)
{
	IDP_ASSERT_VALID(hMessageFactory, IDPCIMPL_MessageFactory)
	try
	{
		*phMessageObject = ((IDPCIMPL_MessageFactory*)hMessageFactory)->CreatePayInit();
		return 0;
	}
	catch(IDP_Exception e)
	{
		*phMessageObject = (IDP_HANDLE)0;
		return e.hr;
	}
}

IDPEXPORT HRESULT MessageFactory_CreatePayComplete(IDP_HANDLE hMessageFactory,
												   const char *pszId,
												   const char *pszToken,
												   IDP_HANDLE *phMessageObject)
{
	IDP_ASSERT_VALID(hMessageFactory, IDPCIMPL_MessageFactory)
	try
	{
		*phMessageObject =
			((IDPCIMPL_MessageFactory*)hMessageFactory)->CreatePayComplete(mbstoSTRING(pszId),
				mbstoSTRING(pszToken));
		return 0;
	}
	catch(IDP_Exception e)
	{
		*phMessageObject = (IDP_HANDLE)0;
		return e.hr;
	}
}

IDPEXPORT HRESULT MessageFactory_Open(IDP_HANDLE hMessageFactory, const char *pszPath)
{
	IDP_ASSERT_VALID(hMessageFactory, IDPCIMPL_MessageFactory)
	try
	{
		((IDPCIMPL_MessageFactory*)hMessageFactory)->Open(mbstoSTRING(pszPath));
		return 0;
	}
	catch(IDP_Exception e)
	{
		return e.hr;
	}
}

IDPEXPORT HRESULT MessageFactory_CreateRequest(IDP_HANDLE hMessageFactory,
												   const char *pszMsgType,
												   IDP_HANDLE *phMessageObject)
{
	IDP_ASSERT_VALID(hMessageFactory, IDPCIMPL_MessageFactory)
	try
	{
		*phMessageObject =
			((IDPCIMPL_MessageFactory*)hMessageFactory)->CreateRequest(mbstoSTRING(pszMsgType));
		return 0;
	}
	catch(IDP_Exception e)
	{
		*phMessageObject = (IDP_HANDLE)0;
		return e.hr;
	}
}

}	// extern "C"



