// cMessageObject.cpp: C-interface for the Saferpay merchant application

#include <stdio.h>

#include "idpapi.h"
#include "idputil.h"
#include "cppMessageObject.h"


void IDPCIMPL_MessageObject::Open(const STRING & Path)
{
	UDEBUG("MessageObject::Open()\n");
	m_ctx.Open(Path);
	m_msg.Create(&m_ctx, _T("ERROR"));
}

void IDPCIMPL_MessageObject::SetAttribute(const STRING & Name, const STRING & Value)
{
	UDEBUG("MessageObject::SetAttribute()\n");
	m_msg.SetAttribute(Name, Value);
}

void IDPCIMPL_MessageObject::GetURL(STRING & pVal)
{
	UDEBUG("MessageObject::GetURL()\n");
	m_msg.GetURL(pVal);

	if(!m_Outgoing)
		IDPTHROW1(IDP_E_UNEXPECTED);
}

void IDPCIMPL_MessageObject::GetAttribute(const STRING & Name, STRING & pVal)
{
	UDEBUG("MessageObject::GetAttribute()\n");
	
	m_msg.GetAttribute(Name, pVal);
}

void IDPCIMPL_MessageObject::Capture()
{
	UDEBUG("MessageObject::Capture()\n");

	m_msg.Capture();
}

IDP_HANDLE IDPCIMPL_MessageObject::Execute()
{
	UDEBUG("MessageObject::Execute()\n");

	IDPCIMPL_MessageObject *Msg = (IDPCIMPL_MessageObject*) CreateMessageObject();

	try
	{
		Msg->m_ctx = m_ctx;
		m_msg.Request(Msg->m_msg);

		return (IDP_HANDLE)Msg;
	}
	catch(IDP_Exception e)
	{
		DestroyMessageObject((IDP_HANDLE)Msg);
		throw e;
		return (IDP_HANDLE)0;
	}
}

void IDPCIMPL_MessageObject::FromString(const STRING & Message)
{
	UDEBUG("MessageObject::FromString()\n");

	/*if(m_ctx.m_Path.empty())
	{
		m_ctx.Open(_T(""));
	}

	if(!m_ctx.QueryValue(_T("keys/current"), _T(""), m_msg.m_KeyId))
		IDPTHROW1(IDP_E_NO_KEYID);*/

	xmlElementPtr pElem;
	m_msg.m_pData = NULL;
	m_ctx.XmlFromString(Message, m_msg.m_pData.Ptr());
	m_msg.SetAttribute(_T("KEYID"), m_msg.m_KeyId);
}

STRING IDPCIMPL_MessageObject::ToString()
{
	UDEBUG("MessageObject::ToString()\n");
		
	STRINGSTREAM str;
	if(!m_msg.m_pData->toStream(str))
		IDPTHROW1(IDP_E_UNEXPECTED);

	return STRING(str.str());
}

// C-interface functions
extern "C" {

// factory
IDPEXPORT void DestroyMessageObject(IDP_HANDLE hMessageObject)
{
	delete (IDPCIMPL_MessageObject*) hMessageObject;
}


IDPEXPORT IDP_HANDLE CreateMessageObject()
{
	IDPCIMPL_MessageObject *hMessageObject = new IDPCIMPL_MessageObject;
	return (IDP_HANDLE)hMessageObject;
}



IDPEXPORT HRESULT MessageObject_GetURL(IDP_HANDLE hMessageObject, long *pcbSize, char *pURL)
{
	IDP_ASSERT_VALID(hMessageObject, IDPCIMPL_MessageObject)
	STRING sURL = _T("");
	try
	{
		((IDPCIMPL_MessageObject*)hMessageObject)->GetURL(sURL);
	}
	catch(IDP_Exception e)
	{
		return e.hr;
	}

	char *buf = new char[sURL.size()*2+2];	// MBCS
	HRESULT hr = 0;
#ifdef UNICODE
	sprintf(buf, "%S", sURL.c_str());
#else
	sprintf(buf, "%s", sURL.c_str());
#endif
	long size = strlen(buf)+1;
	if(pcbSize)
	{
		if(*pcbSize >= size)
		{
			if(pURL) strcpy(pURL, buf);
		}
		else hr = IDP_E_BUFFER_SIZE;
		*pcbSize = size;
	}
	else hr = IDP_E_BUFFER_SIZE;

	delete buf;
	return hr;
}

IDPEXPORT HRESULT MessageObject_SetAttribute(IDP_HANDLE hMessageObject, const char *pszName,
											 const char *pszValue)
{
	IDP_ASSERT_VALID(hMessageObject, IDPCIMPL_MessageObject)
	try
	{
		((IDPCIMPL_MessageObject*)hMessageObject)->SetAttribute(mbstoSTRING(pszName),
			mbstoSTRING(pszValue));
		return 0;
	}
	catch(IDP_Exception e)
	{
		return e.hr;
	}
}

IDPEXPORT HRESULT MessageObject_GetAttribute(IDP_HANDLE hMessageObject, const char *pszName,
											 long *pcbSize, char *pValue)
{
	IDP_ASSERT_VALID(hMessageObject, IDPCIMPL_MessageObject)
	STRING sValue = _T("");
	try
	{
		((IDPCIMPL_MessageObject*)hMessageObject)->GetAttribute(mbstoSTRING(pszName), sValue);
	}
	catch(IDP_Exception e)
	{
		return e.hr;
	}

	char *buf = new char[sValue.size()*2+2];	// MBCS
	HRESULT hr = 0;
#ifdef UNICODE
	sprintf(buf, "%S", sValue.c_str());
#else
	sprintf(buf, "%s", sValue.c_str());
#endif
	long size = strlen(buf)+1;
	if(pcbSize)
	{
		if(*pcbSize >= size)
		{
			if(pValue) strcpy(pValue, buf);
		}
		else hr = IDP_E_BUFFER_SIZE;
		*pcbSize = size;
	}
	else hr = IDP_E_BUFFER_SIZE;

	delete buf;
	return hr;
}

IDPEXPORT HRESULT MessageObject_Capture(IDP_HANDLE hMessageObject)
{
	IDP_ASSERT_VALID(hMessageObject, IDPCIMPL_MessageObject)

	try
	{
		((IDPCIMPL_MessageObject*)hMessageObject)->Capture();
		return 0;
	}
	catch(IDP_Exception e)
	{
		return e.hr;
	}
}

IDPEXPORT HRESULT MessageObject_Execute(IDP_HANDLE hMessageObject, IDP_HANDLE *phResponse)
{
	IDP_ASSERT_VALID(hMessageObject, IDPCIMPL_MessageObject)

	try
	{
		*phResponse = ((IDPCIMPL_MessageObject*)hMessageObject)->Execute();
		return 0;
	}
	catch(IDP_Exception e)
	{
		*phResponse = (IDP_HANDLE)0;
		return e.hr;
	}
}


IDPEXPORT HRESULT MessageObject_Open(IDP_HANDLE hMessageObject, const char *pszPath)
{
	IDP_ASSERT_VALID(hMessageObject, IDPCIMPL_MessageObject)
	try
	{
		((IDPCIMPL_MessageObject*)hMessageObject)->Open(mbstoSTRING(pszPath));
		return 0;
	}
	catch(IDP_Exception e)
	{
		return e.hr;
	}
}

IDPEXPORT HRESULT MessageObject_FromString(IDP_HANDLE hMessageObject, const char * Message)
{
	IDP_ASSERT_VALID(hMessageObject, IDPCIMPL_MessageObject)
	try
	{
		((IDPCIMPL_MessageObject*)hMessageObject)->FromString(mbstoSTRING(Message));
		return 0;
	}
	catch(IDP_Exception e)
	{
		return e.hr;
	}
}

IDPEXPORT HRESULT MessageObject_ToString(IDP_HANDLE hMessageObject, long *pcbSize,
										 char *pValue)
{
	IDP_ASSERT_VALID(hMessageObject, IDPCIMPL_MessageObject)
	STRING sValue = _T("");
	try
	{
		sValue = ((IDPCIMPL_MessageObject*)hMessageObject)->ToString();
	}
	catch(IDP_Exception e)
	{
		return e.hr;
	}

	char *buf = new char[sValue.size()*2+2];	// MBCS
	if(!buf)
		return IDP_E_OUT_OF_MEMORY;

	HRESULT hr = 0;
#ifdef UNICODE
	sprintf(buf, "%S", sValue.c_str());
#else
	sprintf(buf, "%s", sValue.c_str());
#endif
	long size = strlen(buf)+1;
	if(pcbSize)
	{
		if(*pcbSize >= size)
		{
			if(pValue) strcpy(pValue, buf);
		}
		else hr = IDP_E_BUFFER_SIZE;
		*pcbSize = size;
	}
	else hr = IDP_E_BUFFER_SIZE;

	delete buf;
	return hr;
}

}	// extern "C"
