####### Compiler, tools and options

SSLVERSION = openssl-0.9.7b
SSLEAYDIR = ./ssl
SSLINCDIR = -I ./ssl


CC	=	gcc
CXX	=	g++

CFLAGS	=	-pipe -DUNIX -DHAVE_UNISTD_H -DHAVE_STDLIB_H -O2 -fPIC
CXXFLAGS=	-pipe -DUNIX -O2 -fPIC
INCPATH	=	-I"./xml" -I"./jni" -I"./sxfc" $(SSLINCDIR)
LINK	=	g++
LFLAGS	=	-fPIC -shared
LIBS	=	/usr/lib/libssl.a /usr/lib/libcrypto.a
AR	=	ar cq
RANLIB	=	
MOC	=	moc

TAR	=	tar -cf
GZIP	=	gzip -9f

####### Files

HEADERS =	IcctSign.h \
		cppConfigurationSetup.h \
		cppMessageFactory.h \
		cppMessageObject.h \
		idpapi.h \
		idpctx.h \
		idpdattr.h \
		idperr.h \
		idperrc.h \
		idpexcpt.h \
		idphttp.h \
		idpjni.h \
		idptypes.h \
		idpurl.h \
		idputil.h \
		jConfigurationSetup.h \
		jMessageFactory.h \
		jMessageObject.h \
		jni/jni.h \
		jni/jni_md.h \
		spliturl.h \
		sxfc/sxcontainer.h \
		sxfc/sxexcept.h \
		sxfc/sxfcerr.h \
		sxfc/sxfctype.h \
		sxfc/sxfctypes.h \
		sxfc/sxlist.h \
		sxfc/sxmap.h \
		sxfc/sxobject.h \
		sxfc/sxpair.h \
		sxfc/sxptr.h \
		sxfc/sxsort.h \
		sxfc/sxstack.h \
		sxfc/sxstream.h \
		sxfc/sxstring.h \
		sxfc/sxtchar.h \
		sxfc/sxvector.h \
		uuid.h \
		uuidP.h \
		xml/sxstream.h \
		xml/xmlElement.h \
		xml/xmlElementEnum.h \
		xml/xmlParser.h \
		xml/xmlcore.h \
		xml/xmlerror.h \
		xml/xmltab.h \
		uuid.h \
		uuidP.h
SOURCES =	IcctSign.cpp \
		cConfigurationSetup.cpp \
		cMessageFactory.cpp \
		cMessageObject.cpp \
		idpbase.cpp \
		idpctx.cpp \
		idpdattr.cpp \
		idperr.cpp \
		idphttp.cpp \
		idpurl.cpp \
		idputil.cpp \
		jConfigurationSetup.cpp \
		jMessageFactory.cpp \
		jMessageObject.cpp \
		spliturl.cpp \
		sxfc/sxmap.cpp \
		sxfc/sxobject.cpp \
		sxfc/sxpair.cpp \
		sxfc/sxstring.cpp \
		sxfc/sxtchar.c \
		sxfc/sxvector.cpp \
		xml/xmlElement.cpp \
		xml/xmlElementEnum.cpp \
		xml/xmlParser.cpp \
		xml/xmltab.c \
		guid.c \
		uuid_time.c \
		pack.c \
		unpack.c
OBJECTS =	IcctSign.o \
		cConfigurationSetup.o \
		cMessageFactory.o \
		cMessageObject.o \
		idpbase.o \
		idpctx.o \
		idpdattr.o \
		idperr.o \
		idphttp.o \
		idpurl.o \
		idputil.o \
		jConfigurationSetup.o \
		jMessageFactory.o \
		jMessageObject.o \
		spliturl.o \
		sxfc/sxmap.o \
		sxfc/sxobject.o \
		sxfc/sxpair.o \
		sxfc/sxstring.o \
		sxfc/sxtchar.o \
		sxfc/sxvector.o \
		xml/xmlElement.o \
		xml/xmlElementEnum.o \
		xml/xmlParser.o \
		xml/xmltab.o \
		guid.o \
		uuid_time.o \
		pack.o \
		unpack.o
SRCMOC	=	
OBJMOC	=	
DIST	=	
TARGET	=	libidpapp.sl
TARGETA	=	./out/libidpapp.a
TARGETD	=	libidpapp.sl
TARGET0	=	libidpapp.so
TARGET1	=	libidpapp.so.1
TARGET2	=	libidpapp.so.1.0
TARGET3 =	libidpapp.so.1.0.0

####### Implicit rules

.SUFFIXES: .cpp .cxx .cc .C .c

.cpp.o:
	$(CXX) -c $(CXXFLAGS) $(INCPATH) -o $@ $<

.cxx.o:
	$(CXX) -c $(CXXFLAGS) $(INCPATH) -o $@ $<

.cc.o:
	$(CXX) -c $(CXXFLAGS) $(INCPATH) -o $@ $<

.C.o:
	$(CXX) -c $(CXXFLAGS) $(INCPATH) -o $@ $<

.c.o:
	$(CC) -c $(CFLAGS) $(INCPATH) -o $@ $<

####### Build rules

all: ./out/libidpapp.sl

./out/libidpapp.sl: $(OBJECTS) $(OBJMOC) 
	-rm -f $(TARGET) $(TARGET0) $(TARGET1) $(TARGET2)
	$(LINK) $(LFLAGS) -o $(TARGETD) $(OBJECTS) $(OBJMOC) $(LIBS)
	-ln -s $(TARGET) $(TARGET0)
	-ln -s $(TARGET) $(TARGET1)
	-ln -s $(TARGET) $(TARGET2)
	-ln -s $(TARGET) $(TARGET3)
	-rm -f ./out/$(TARGET)
	-rm -f ./out/$(TARGET0)
	-rm -f ./out/$(TARGET1)
	-rm -f ./out/$(TARGET2)
	-rm -f ./out/$(TARGET3)
	-mv $(TARGET) $(TARGET0) $(TARGET1) $(TARGET2) $(TARGET3) ./out/


staticlib: $(TARGETA)

$(TARGETA): $(OBJECTS) $(OBJMOC) 
	-rm -f $(TARGETA)
	$(AR) $(TARGETA) $(OBJECTS) $(OBJMOC)

moc: $(SRCMOC)

#no more tmake
tmake: idpapp.mk

idpapp.mk: idpapp.pro
	tmake idpapp.pro -o idpapp.mk

dist:
	$(TAR) idpapp.tar idpapp.pro $(SOURCES) $(HEADERS) $(DIST)
	$(GZIP) idpapp.tar

clean:
	-rm -f $(OBJECTS) $(OBJMOC) $(SRCMOC) $(TARGET)
	-rm -f $(TARGET0) $(TARGET1) $(TARGET2) $(TARGETA)
	-rm -f *~ core
	-rm -f ./out/libidpapp*

####### Compile

IcctSign.o: IcctSign.cpp

cConfigurationSetup.o: cConfigurationSetup.cpp

cMessageFactory.o: cMessageFactory.cpp

cMessageObject.o: cMessageObject.cpp

idpbase.o: idpbase.cpp

idpctx.o: idpctx.cpp

idpdattr.o: idpdattr.cpp

idperr.o: idperr.cpp

idphttp.o: idphttp.cpp

idpurl.o: idpurl.cpp

idputil.o: idputil.cpp

jConfigurationSetup.o: jConfigurationSetup.cpp

jMessageFactory.o: jMessageFactory.cpp

jMessageObject.o: jMessageObject.cpp

spliturl.o: spliturl.cpp

sxfc/sxmap.o: sxfc/sxmap.cpp

sxfc/sxobject.o: sxfc/sxobject.cpp

sxfc/sxpair.o: sxfc/sxpair.cpp

sxfc/sxstring.o: sxfc/sxstring.cpp

sxfc/sxtchar.o: sxfc/sxtchar.c

sxfc/sxvector.o: sxfc/sxvector.cpp

xml/xmlElement.o: xml/xmlElement.cpp

xml/xmlElementEnum.o: xml/xmlElementEnum.cpp

xml/xmlParser.o: xml/xmlParser.cpp

xml/xmltab.o: xml/xmltab.c

guid.o: guid.c

uuid_time.o: uuid_time.c

pack.o: pack.c

unpack.o: unpack.c

