// idpbase.cpp


#ifdef WIN32
#include "stdafx.h"
#include "CreateFullPath.h"
#endif

#include "idpctx.h"

#ifdef UNIX
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <dirent.h>
#include <errno.h>
#include <stdio.h>

#include "uuid.h"

#define MAX_PATH 255
#define MAX_DIR_NAME 255
#endif

LPCTSTR IDP_DllName = _T("idpapp.dll");

void IDP_Context::CreatePath(const STRING & Path)
{
	// WINDOWS
#ifdef WIN32
	if(!CreateFullPath(Path.c_str()))
	{
		IDPTHROW1(HRESULT_FROM_WIN32(GetLastError()));
		return;
	}
#endif // WIN32

	// UNIX
#ifdef UNIX
	char buf[MAX_PATH];
	mode_t m;
	DIR *pDir = NULL;

#ifdef UNICODE
	wcstombs(buf, Path.c_str(), Path.size());
#else
	strncpy(buf, Path.c_str(), Path.size());
#endif
	buf[Path.size()] = char(0);
	if((pDir = opendir(buf)) == NULL)
	{
		if(mkdir(buf, 0777))
		{
			IDPTHROW2(errno, _T("IDP_Context::CreatePath(): failed to create directory"));
			return;
		}
	}
	else
		closedir(pDir);
#endif // UNIX
	TCHAR msg[MAX_PATH];
	tsprintf(msg, _T("CREATE PATH: %s"), Path.c_str());
	LogMessage(msg);
}



STRING IDP_Context::GetDefaultConfiguration()
{
	// WINDOWS
#ifdef WIN32
	HMODULE hMod = GetModuleHandle(IDP_DllName);
	if(hMod == NULL)
		IDPTHROW1(HRESULT_FROM_WIN32(GetLastError()));

	TCHAR ModuleName[MAX_PATH];
	if(!GetModuleFileName(hMod, ModuleName, sizeof(ModuleName)))
		IDPTHROW1(HRESULT_FROM_WIN32(GetLastError()));

	TCHAR * pPath = 0;

#ifdef UNICODE
	pPath = wcsrchr(ModuleName, L'\\');
#else
	pPath = strrchr(ModuleName, '\\');
#endif

	if(!pPath)
		IDPTHROW1(IDP_E_NO_DEFAULT_PATH);

	* pPath = 0;
		
	STRING r = ModuleName;
	return r;
#endif // WIN32


	// UNIX
#ifdef UNIX
	STRING Path = _T("");
	char buf[MAX_DIR_NAME+1];
	buf[MAX_DIR_NAME] = '\0';

	getcwd(buf, MAX_DIR_NAME);
	Path = buf;

	return Path;
#endif // UNIX
}

STRING IDP_Context::CreateUuid()
{

	GUID guid;

#ifdef WIN32
	CoCreateGuid(&guid);
#else
	uuid_generate(&guid);
#endif

	TCHAR s[64]; TCHAR * p = s;
	p += tsprintf(p, _T("%08x"), guid.Data1);
	p += tsprintf(p, _T("%04x"), guid.Data2);
	p += tsprintf(p, _T("%04x"), guid.Data3);
	for(int i=0; i<8; ++i)

		p += tsprintf(p, _T("%02x"), guid.Data4[i]);

	return s;
}


