// idpctx.h

#ifndef __IDPCTX_H__
#define __IDPCTX_H__

#include "idptypes.h"
#include "idperr.h"
#include "xmlElement.h"

#ifdef _DEBUG
#define UDEBUG(str) printf(str)
#else
#define UDEBUG
#endif

class IDP_Message;

class IDP_Context
{
public:
	void Open(const STRING & Path);

	void LoadDefaults();
	void RequestToURL(const STRING & BaseURL, STRINGSTREAM & Request, const STRING Signature, STRING & Url);
	void GenKey(STRING & KeyId);
	void SignData(const STRING & Data, const STRING & KeyId, STRING & Signature);
	void SetOption(const STRING & OptionName, const STRING & Value);
	void LogMessage(const STRING & msg);
	void CreatePath(const STRING & Path);
	void StreamFromURL(const STRING & url, STRINGSTREAM & s);
	STRING CreateUuid();
	STRING GetDefaultConfiguration();
	STRING GetPath();
	void VerifyData(const STRING & Data, const STRING & KeyId, const STRING & Signature);
	bool QueryValue(const STRING & SubPath, const STRING & SubItem, STRING & Result);
	bool QueryValue(const STRING & SubPath, UCHAR * pData, ULONG * pcbData);
	void SetValue(const STRING & SubPath, const STRING & SubItem, const STRING & Result, bool CreateFile = false);
	void CheckResponseMsgType(xmlElement * pElem, const STRING & MsgType);

private:
	void UpdateFile(const STRING & SubPath, const STRING & Data);
	void UpdateFile(const STRING & SubPath, UCHAR * pData, ULONG cbData);

public:
	// http roundtrips
	void PublishKey(STRING & KeyId);
	void ImportKey(STRING & KeyId);
	void Setup();

	// public helpers

	static void XmlFromStream(STRINGSTREAM & s, xmlElement ** ppElem);
	static HRESULT IDPCALL XmlCreateRequest(const STRING & MsgType, xmlElement ** ppElem);
	static void XmlFromString(const STRING & s, xmlElement ** ppElem);

public:
	// major operations
	void Reconfigure();

private:
	void SlToBackSl(STRING & s);
	STRING Encode(const STRING & a);
	STRING BufferToHex(UCHAR * pData, ULONG cbData);
	HRESULT HexToBuffer(STRING r, UCHAR * data, ULONG * size);

private:
	STRING m_Path;
	STRING m_UserName;
	STRING m_Password;
	STRING m_VtAutoURL;
};

class IDP_Message
{
public:
	void Create(IDP_Context * pCtx, const STRING & MsgType);
	void SetAttribute(const STRING & Attribute, const STRING & Value);
	void GetURL(STRING & Url);
	void Open(IDP_Context * pCtx, const STRING & Data, const STRING & Signature);
	void GetAttribute(const STRING & Attribute, STRING & Value);
	void Capture();
	void Request(IDP_Message & Response);

public:
	xmlElementPtr m_pData;
	IDP_Context * m_pCtx;
	STRING m_KeyId;
};

class IDP_ErrorLookup
{
public:
	static bool GetErrorStringFromResult(HRESULT hr, STRING & txt);
	static bool UseExtendedErrorInfo();
};


#endif	// defined(__IDPCTX_H__)
