/***************************************************************************
 *
 *
 * TITLE			idphttp.cpp
 *
 * AUTHOR			Thomas Cerny/ Markus Gagliardi
 *
 * COMPANY		Soplex AG
 *
 * COPYRIGHT	(c) 1999 soplex ag
 *
 * CREATION		08.12.99
 *
 * MODIFIED			
 *
 * VERSION			1.00.000
 *
 * PURPOSE			http helper functions
 *
 * LANGUAGE			C++
 *
 * PROCESSING		gcc, Microsoft Visual C++
 *
 * PLATTFORM		
 *
 ***************************************************************************/

// debug and network settings
//#define _HTTP_TRACE		// to see http data on stdout
#define ICCT_USE_SSL		// to enable SSL-protocol



// WIN32 socket
#ifdef WIN32
// windows specific
#include <windows.h>
WSADATA wsd;
#define _ICONN_ERR WSAGetLastError()
#endif //WIN32

// ANSI
#include <stddef.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>

// UNIX sockets
#ifdef UNIX
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#ifndef HOSTENT
#define HOSTENT hostent
#endif
#ifndef INADDR_NONE
#define INADDR_NONE             0xffffffff
#endif
#ifndef INVALID_SOCKET
#define INVALID_SOCKET			(-1)
#endif
typedef struct sockaddr SOCKADDR;
typedef struct sockaddr_in SOCKADDR_IN;
typedef int SOCKET;

#define _ICONN_ERR errno
#endif // UNIX


// SSL library

// PSDK workaround
#undef X509_NAME
#undef PKCS7_SIGNER_INFO

#include "rsa.h"
#include "crypto.h"
#include "x509.h"
#include "pem.h"
#include "ssl.h"
#include "err.h"

#define SSL090
#ifdef SSL090
	//#define SSL_CTX_new() SSL_CTX_new(SSLv3_client_method())
#endif

#include "idptypes.h"
#include "idphttp.h"
#include "spliturl.h"
#include "idputil.h"

#include "sxfc/sxstream.h"


//	internal class definitions
//	--------------------------

class IDP_Connection
{
public:
	virtual ~IDP_Connection() { }

public:
	virtual int Connect(SOCKADDR * sa) = 0;
	virtual int Disconnect() = 0;
	virtual int Read(SxStringStream & out) = 0;
	virtual int Write(SxStringStream & in) = 0;
	virtual int LastError() = 0;
	virtual void Attach(SOCKET s) = 0;
	virtual SOCKET Detach() = 0;
	virtual int Setup() = 0;
};

class IDP_HttpSession
{
private:
	int m_state;
	const URL_INFO & m_ui;
	IDP_Connection * m_c;
	SxStringStream m_request;
	SxStringStream m_response;
	static LPCTSTR m_UserAgent;

public:
	IDP_HttpSession(const URL_INFO & ui) : m_ui(ui) { m_state = 0; m_c = 0; }
	virtual ~IDP_HttpSession();

	void ConnectNetwork();
	void BuildRequest();
	void SendRequest();
	void ReceiveResponse();
	void ExtractResponse(SxStringStream & out);

private:
	void ProxyConnect();
};

LPCTSTR IDP_HttpSession::m_UserAgent = _T("Saferpay Application Component 1.0");

//	base 64 encoding
//	----------------

TCHAR base64_table[] = _T("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/");

/*int base64_index(char c)
{
	printf("('%c')", c);
	char * p = strchr(base64_table, c);
	if(!p) return EOF;

	return (p - base64_table);
}

void Base64Decode(SxStringStream * in, SxStringStream * out)
{
	int state = 0;
	unsigned char a, c;

	while(!in->eof())
	{
		char _c;
		in->get(_c);
		c = _c;

		// lookup tag

		c = base64_index(c);

		switch(state)
		{
		case 0:
			a = c << 2;			// first 6 bits of first byte
			state = 1;
			break;

		case 1:
			a |= c >> 4;		// last 2 bits of first byte
			* out << a;
			a = c << 4;			// first 4 bits of second byte
			state = 2;
			break;

		case 2:
			a |= c >> 2;		// last 4 bits of second byte
			* out << a;
			a = (c & 3) << 6;	// first 2 of third byte
			state = 3;
			break;

		case 3:
			a |= c;				// last 6 bits of third byte
			* out << a;
			state = 0;
			break;
		}
	}
}*/

void Base64Encode(STRINGSTREAM * in, STRINGSTREAM * out)
{
	int state = 0;
	TCHAR tc;
	unsigned char c, a; char pc; int eof = 0;

	while(!((state == 0) && eof))
	{
		if(!in->eof())
		{
			in->get(tc);
			pc = (char)tc;
		}

		if((pc <= 0) || (in->eof()))
		{
			pc = 0;
			eof = 1;
		}

		c = (unsigned char) pc;

		switch(state)
		{
		case 0:
			a = c >> 2;
			* out << base64_table[a];
			a = (c & 3) << 4;
			state = 1;
			break;

		case 1:
			a |= c >> 4;
			* out << base64_table[a];
			a = (c & 0x0f) << 2;
			state = 2;
			break;

		case 2:
			a |= c >> 6;
			* out << base64_table[a];
			a = (c & 0x3f);
			* out << base64_table[a];
			state = 0;
		}
	}
}

//	URL encoding
//	------------

void UrlEncode(SxStringStream * in, SxStringStream * out)
{
	unsigned char c;
	while(1)
	{	
		TCHAR _tc;
		UCHAR _c;
		in->get(_tc);
		_c = (UCHAR)_tc;
		c = _c;

		if(in->eof() || !c)
		{
			return;
		}


		if(!isalnum(c))
		{
			char buf[100]; sprintf(buf, "%%%02x\0", c);
			* out << mbstoSTRING(buf);
		}
		else
		{
			* out << (TCHAR) c;
		}
	}
}

// UNICODE helper class
class IDP_CHARBuffer
{
private:
	char *p;

public:
	IDP_CHARBuffer() { p = 0; }
	IDP_CHARBuffer(const SxString& str)
	{
		p = new char[(str.size() + 1) * sizeof(TCHAR)];
		if(!p)
			return;
#ifdef UNICODE
		WCSToMBS(p, str.c_str(), str.size());
		p[str.size()] = (TCHAR)0;
#else
		strcpy(p, str.c_str());
#endif
	}

	~IDP_CHARBuffer() { if(!!p) delete p; }
	void Allocate(SX_SIZE_T Size) { if(!!p) delete p; if(Size) p = new char[Size]; }
	char *DataPtr() { _ASSERT(!!p); return p; }
};



void HttpRequest(const char * uri,
				 const char * username,
				 const char * password,
				 SxStringStream * out)
{
	SxStringStream r;
	r << mbstoSTRING(uri) << TCHAR(0);

	SxStringStream t, v;
	t << mbstoSTRING(username) << _T(":") << mbstoSTRING(password);
	Base64Encode(&t, &v); v << TCHAR(0);

	* out <<	_T("GET ") << r.str() << _T(" HTTP/1.0\r\n")
		_T("Authorization: Basic ") << v.str() << _T("\r\n\r\n");
}

unsigned long HttpHostByName(const char * szHost)
{
	HOSTENT * ph;
	ph = gethostbyname(szHost);

	if(!ph) return INADDR_NONE;

	char ** p_addrs = ph->h_addr_list;
	if(! * p_addrs)	
	{
		IDPTRACE(IDP_HTTP_DNS_ERROR, _T("DNS LOOKUP"), _T("failed to lookup host"));
		return INADDR_NONE;
	}

	UCHAR * pa = (UCHAR *) * p_addrs;
	STRINGSTREAM t;
	t << mbstoSTRING(szHost) << _T(" -> ") << int(pa[0]) << _T(".") << int(pa[1]) 
		<< _T(".") << int(pa[2]) << _T(".") << int(pa[3]);

	IDPTRACE(0, _T("DNS LOOKUP"), t.str());

	return * (unsigned long *) * p_addrs;
}


/***************************************************************************
 *
 * SslConnection
 *
 * connection using ssl protocol
 *
 ***************************************************************************/


class SslConnection : public IDP_Connection
{
private:
	struct sockaddr_in sa;
	int sd;
	int err;
	char * str;
	char	buf[1024 + 1];
	SSL_CTX* ctx;
	SSL*     ssl;
	X509*    server_cert;

public:
	SslConnection()
	{
		SSLeay_add_ssl_algorithms();
		SSL_load_error_strings();

		ctx = SSL_CTX_new(SSLv23_method());

		sd = INVALID_SOCKET;
		ssl = NULL;
	}

	~SslConnection()
	{
		SSL_CTX_free(ctx);
	}

	void Attach(SOCKET s)
	{
		sd = s;
	}

	SOCKET Detach()
	{
		SOCKET s = sd; sd = INVALID_SOCKET; return s;
	}

	int Connect(SOCKADDR * sa)
	{
		sd = socket (AF_INET, SOCK_STREAM, 0);
		if(sd == INVALID_SOCKET)
			return err;

		err = connect(sd, sa, sizeof(SOCKADDR));
		
		if(err == -1)
		{
			return LastSocketError();
		}

		return 0;
	}

	int Setup()
	{
		IDPTRACE(0, _T("USE SSL"), _T(""));

		ssl = SSL_new (ctx);
		SSL_set_fd (ssl, sd);
		err = SSL_connect (ssl);

		if(err == -1)
		{
			printf("SSL ERROR: connect failed\n");
			return LastError();
		}
		return 0;
	}

	int Disconnect()
	{
		shutdown (sd, 1);

#ifdef WIN32
		closesocket(sd);
#endif

		if(ssl) SSL_free(ssl);
		return 0;
	}

	int Read(SxStringStream & out)
	{
		err = SSL_read(ssl, buf, 1024);
		if(err == -1)
		{
			return err;
		}
		buf[err] = 0;
		out << mbstoSTRING(buf);
		
		return err;
	}

	int Write(SxStringStream & in)
	{
		IDP_CHARBuffer sz(in.str());
		err = SSL_write(ssl, sz.DataPtr(), in.str().size());

		if(err == -1)
		{
			return LastError();
		}
		return 0;
	}

	int LastError()
	{
		if(ssl) return (IDP_SSL_BASE | ssl->error_code);
		else return LastSocketError();
	}

	int LastSocketError()
	{
		int r;

	#ifdef WIN32
		r = WSAGetLastError();
		if((r / 1000) == 10)
			return ((r - 10000) | IDP_SOCKET_BASE);
		else
			return HRESULT_FROM_WIN32(r);
	#endif

	#ifdef UNIX
		return (errno | IDP_SOCKET_BASE);
	#endif

	}
};


/***************************************************************************
 *
 * SockConnection
 *
 * connection using open socket stream
 *
 ***************************************************************************/

class SockConnection : public IDP_Connection
{
private:
	struct sockaddr_in sa;
	int sd;
	int err;
	char	buf[1024 + 1];

public:
	SockConnection()
	{
		sd = -1;
	}

	~SockConnection()
	{
	}

	void Attach(SOCKET s)
	{
		sd = s;
	}

	SOCKET Detach()
	{
		SOCKET s = sd; sd = INVALID_SOCKET; return s;
	}

	int Setup()
	{
		return 0;
	}

	int Connect(SOCKADDR * sa)
	{
		//std::cout << "[ICCT] socket startup..." << std::endl;

		sd = socket (AF_INET, SOCK_STREAM, 0);


		err = connect(sd, sa, sizeof(SOCKADDR));
		
		if(err == -1)
		{
			err = LastError();
			/*std::cout << "SOCKET: connection failed (" << (err - IDP_SOCKET_BASE) << ")" << std::endl;*/
			return err;
		}

		return 0;
	}

	int Disconnect()
	{
		shutdown (sd, 1);

#ifdef WIN32
		closesocket(sd);
#endif

		return 0;
	}

	int Read(SxStringStream & out)
	{
		err = recv(sd, buf, 1024, 0);

		if(err == -1)
		{
			return err;
		}
		buf[err] = 0;
		out << mbstoSTRING(buf);

		return err;
	}

	int Write(SxStringStream & in)
	{
		IDP_CHARBuffer buf(in.str());

		err = send(sd, buf.DataPtr(), in.str().size(), 0);

		if(err == -1)
		{
			return err;
		}
		return err;
	}

	int LastError()
	{
		int r;

	#ifdef WIN32
		r = WSAGetLastError();
		if((r / 1000) == 10)
			return ((r - 10000) | IDP_SOCKET_BASE);
		else
			return HRESULT_FROM_WIN32(r);
	#endif

	#ifdef UNIX
		return (errno | IDP_SOCKET_BASE);
	#endif

	}
};

/***************************************************************************
 *
 * HttpSession
 *
 * run http query against server
 *
 ***************************************************************************/


int MakeHostAddr(const STRING & Host, const STRING & Port, SOCKADDR * a)
{
	SOCKADDR_IN * sa = (SOCKADDR_IN *) a;

	ULONG t = inet_addr(IDP_CHARBuffer(Host).DataPtr());
	if(t == INADDR_NONE)
	{
		t = HttpHostByName(IDP_CHARBuffer(Host).DataPtr());
		if(t == INADDR_NONE)
			return IDP_HTTP_DNS_ERROR;
	}
	sa->sin_addr.s_addr = t;
	sa->sin_port = htons(atoi(IDP_CHARBuffer(Port).DataPtr()));
	sa->sin_family = AF_INET;

	return 0;
}

//	IDP_HttpSession functions
//	-------------------------

void IDP_HttpSession::ConnectNetwork()
{
	if(m_state != 0)
		IDPTHROW2(IDP_HTTP_INVALID_STATE, _T("invalid session state during ConnectNetwork"));

	// build connection address
	SOCKADDR sa;
	memset(&sa, 0, sizeof(sa));

	// proxy flag
	bool UseProxyConnect = false;
	int r;

	// check if using proxy
	if(!m_ui.ProxyHost.empty())
	{
		r = MakeHostAddr(m_ui.ProxyHost, m_ui.ProxyPort, &sa);
		if(r)
			IDPTHROW2(r, _T("the proxy address is invalid or could not be resolved"));

		STRINGSTREAM t;
		t << m_ui.ProxyHost << _T(":") << m_ui.ProxyPort;
		IDPTRACE(0, _T("USE PROXY"), t.str());

		if(m_ui.Protocol == _T("https"))
		{
			UseProxyConnect = true;
			m_c = new SockConnection;
		}
		else if(m_ui.Protocol == _T("http"))
		{
			m_c = new SockConnection;
		}
		else
			IDPTHROW2(IDP_HTTP_INVALID_PROTOCOL, _T("the protocol type of the URL is unknown"));
	}
	else
	{
		r = MakeHostAddr(m_ui.Host, m_ui.Port, &sa);
		if(r)
			IDPTHROW2(r, _T("the host address is invalid or could not be resolved"));

		// might use SSL for proxy connection
		if(m_ui.Protocol == _T("https"))
		{
			m_c = new SslConnection;
		}
		else if(m_ui.Protocol == _T("http"))
		{
			m_c = new SockConnection;
		}
		else
			IDPTHROW2(IDP_HTTP_INVALID_PROTOCOL, _T("the protocol type of the URL is unknown"));
	}
	
	// connect
	r = m_c->Connect(&sa);
	if(r)
		IDPTHROW2(r, _T("failed to connect to host"));

	if(UseProxyConnect)
	{
		ProxyConnect();
	}

	r = m_c->Setup(); 
	if(r) 
		IDPTHROW2(r, _T("failed to setup protocol")); 

	m_state = 1;
}

void IDP_HttpSession::ProxyConnect()
{
	STRINGSTREAM request, response;

	IDPTRACE(0, _T("HTTP CONNECT"), _T(""));

	request << _T("CONNECT ") << m_ui.Host << _T(":") << m_ui.Port << _T(" HTTP/1.0\r\n");
	request << _T("Pragma: no-cache\r\n");
	request << _T("UserAgent: ") << m_UserAgent << _T("\r\n");
	request << _T("\r\n");


	if(m_c->Write(request) < 0)
		IDPTHROW1(m_c->LastError());

	if(m_c->Read(response) < 0)
		IDPTHROW1(m_c->LastError());
 
	STRING t3 = response.str();

	fprintf(stderr, "Proxy response:\n%s\n", t3.c_str());

	if(t3.find_first_of(_T("HTTP/1")) == SxString::npos)
		IDPTHROW1(IDP_HTTP_INVALID_RESPONSE);

	STRING::size_type nSpace = t3.find_first_of(_T(" "));

	while((t3[nSpace] == ' ') || (t3[nSpace] == '\t'))
		nSpace++;

	STRING t4 = t3.substr(nSpace, 3);

	int r = ttoi(t4.c_str());
	if(r == 0)
		IDPTHROW1(IDP_HTTP_INVALID_RESPONSE);

	if(r != 200)
		IDPTHROW1(IDP_HTTP_BASE | r);

	// exchange the connection object
	SslConnection * c = new SslConnection;
	c->Attach(m_c->Detach());

	delete m_c;
	m_c = c;
}

void IDP_HttpSession::BuildRequest()
{
	if(m_state != 1)
		IDPTHROW2(IDP_HTTP_INVALID_STATE, _T("invalid session state during BuildRequest"));

	m_request << _T("GET ");

	if(!m_ui.ProxyHost.empty() && (m_ui.Protocol != _T("https")))
	{
		m_request << m_ui.Protocol << _T("://") << m_ui.Host << _T(":") << m_ui.Port;
	}

	m_request << m_ui.Uri << _T(" HTTP/1.0\r\n");

	STRINGSTREAM Authorization, EncodedAuthorization;
	Authorization << m_ui.UserName << _T(":") << m_ui.Password;

	Base64Encode(&Authorization, &EncodedAuthorization);

	m_request << _T("Authorization: Basic ") << EncodedAuthorization.str() << _T("\r\n");
	m_request << _T("UserAgent: ") << m_UserAgent << _T("\r\n");
	m_request << _T("\r\n");


	m_state = 2;

	if(m_ui.TraceOpt.find_first_of(_T("rawhttp")) != SxString::npos)
	{
		printf("\n%s\n\n", IDP_CHARBuffer(m_request.str()).DataPtr());
	}
}

void IDP_HttpSession::SendRequest()
{
	if(m_state != 2)
		IDPTHROW2(IDP_HTTP_INVALID_STATE, _T("invalid session state during SendRequest"));

	int cb;
	cb = m_c->Write(m_request);
	if(cb < 0)
		IDPTHROW2(m_c->LastError(), _T("failed to write to host"));

	m_state = 3;
}

void IDP_HttpSession::ReceiveResponse()
{
	if(m_state != 3)
		IDPTHROW2(IDP_HTTP_INVALID_STATE, _T("invalid session state during ReceiveResponse"));

	int cb;
	while(1)
	{
		cb = m_c->Read(m_response);

		if(cb < 0) 
		{ 
			int r;
			r = m_c->LastError(); 
			IDPTHROW2(r, _T("failed to receive from host"));
		}
		if(cb == 0) break;
	}

	m_state = 4;
}

void IDP_HttpSession::ExtractResponse(SxStringStream & out)
{
	if(m_state != 4)
		IDPTHROW2(IDP_HTTP_INVALID_STATE, _T("invalid session state during ReceiveResponse"));

	if(m_ui.TraceOpt.find_first_of(_T("rawhttp")) != SxString::npos)
	{
		printf("\n%s\n\n", IDP_CHARBuffer(m_response.str()).DataPtr());
	}


	const char * p;
	IDP_CHARBuffer s(m_response.str());
	p = s.DataPtr();

	p = strchr(p, ' ');
	if(!p)
		IDPTHROW1(IDP_HTTP_INVALID_RESPONSE);

	p++;

	int r = atoi(p);
	if(r == 0)
		IDPTHROW1(IDP_HTTP_INVALID_RESPONSE);

	if(r != 200)
		IDPTHROW1(IDP_HTTP_BASE | r);

	p = strstr(p, "\r\n\r\n");
	if(!p)
		IDPTHROW1(IDP_HTTP_INVALID_RESPONSE);

	p += 4;
	out << mbstoSTRING(p);

}

IDP_HttpSession::~IDP_HttpSession()
{
	if(m_c)
	{
		m_c->Disconnect();
		delete m_c;
	}
}


void HttpSession(const URL_INFO & ui, SxStringStream & out)
{
	IDP_HttpSession session(ui);
	// split url first

	STRINGSTREAM t;
	t << ui.Protocol << _T("://") << ui.Host
		<< _T(":") << ui.Port;

	IDPTRACE(0, _T("URL OPEN"), t.str());

	session.ConnectNetwork();
	session.BuildRequest();
	session.SendRequest();
	session.ReceiveResponse();
	session.ExtractResponse(out);
}

/***************************************************************************
 *
 * HttpInit
 *
 * initialize http (windows only)
 *
 ***************************************************************************/

#ifdef WIN32

class IDP_HttpInit
{
public:
	IDP_HttpInit()
	{
		// startup winsock
		WSAStartup(MAKEWORD(1, 0), &wsd);
	}

	~IDP_HttpInit()
	{
		WSACleanup();
	}

}
the_idp_HttpInit;

#endif

