// idpurl.cpp

#include <time.h>
#include <stdio.h>
#include <string.h>
#include "idpctx.h"

#ifndef WIN32
#include "uuid.h"
#endif

#ifdef WIN32
typedef WCHAR w_char;
#endif

#include "idphttp.h"
#include "idperr.h"
#include "idpurl.h"
#include "idputil.h"

template <class _Itf>
class IDP_SafePtr
{
private:
	_Itf * p;
public:
	IDP_SafePtr() { p = 0; }
	~IDP_SafePtr() { if(!!p) { p->Release(); } }
	_Itf ** operator & () { _ASSERT(!p); return & p; };

	_Itf * operator -> () { _ASSERT(!!p);  return p; }
};



long IsHttp(const STRING & url)
{
	const TCHAR *p = url.c_str();
	if(tcsncmp(p, _T("http:"), 5) == 0 || tcsncmp(p, _T("https:"), 5) == 0)
		return 1;
	else return 0;
}


void IDP_Context::StreamFromURL(const STRING & url, STRINGSTREAM & s)
{
	if(IsHttp(url))
	{
		URL_INFO ui; 
		if(!SplitUrl(url, ui))
			IDPTHROW1(IDP_HTTP_INVALID_URL);

		STRING UseProxy;
		if(QueryValue(_T("settings.xml"), _T("USEPROXY"), UseProxy))
		{
			if(UseProxy == _T("1"))
			{
				if(QueryValue(_T("settings.xml"), _T("PROXYHOST"), ui.ProxyHost))
				{
					if(!QueryValue(_T("settings.xml"), _T("PROXYPORT"), ui.ProxyPort))
					{
						ui.ProxyPort = _T("8080");
					}
				}
			}
		}

		QueryValue(_T("settings.xml"), _T("TRACEOPT"), ui.TraceOpt);

		HttpSession(ui, s);
	
	}
	else
	{
#ifdef UNICODE
		char sURL[256];
		WCSToMBS(sURL, url.c_str());
		FILE *pFile = fopen(sURL, "rb");
#else
		FILE *pFile = fopen(url.c_str(), "rb");
#endif
		if(!pFile)
		{
			IDPTHROW2(IDP_E_RESOURCE_NOT_FOUND, url);
			return;
		}
		int c;
		while((c = fgetc(pFile)) != EOF)
			s << TCHAR(c);
		fclose(pFile);
	}
}

