// jConfigurationSetup.cpp

#include <stdio.h>

#include "idputil.h"
#include "idpapi.h"
#include "cppConfigurationSetup.h"
#include "jConfigurationSetup.h"
#include "idpjni.h"


extern "C" {

JNIEXPORT void JNICALL Java_soplex_saferpay_ConfigurationSetup_nConstruct
  (JNIEnv *env, jobject obj)
{
	jfieldID fid;
	jint Handle;

	jclass cls = env->GetObjectClass(obj);

	fid = env->GetFieldID(cls, "m_Handle", "I");
	if (fid == 0)
	{
		IDPJNI_THROW1("ConfigurationSetup::Construct(): member m_Handle not found",
			IDP_E_UNEXPECTED)
		return;
	}
	Handle = env->GetIntField(obj, fid);
	if(Handle)
	{
		IDPJNI_THROW1("ConfigurationSetup::Construct(): m_Handle != 0",
			IDP_E_UNEXPECTED)
		return;
	}
	env->SetIntField(obj, fid, (jint)CreateConfigurationSetup());
}

JNIEXPORT void JNICALL Java_soplex_saferpay_ConfigurationSetup_nDestruct
  (JNIEnv *env, jobject obj)
{
	IDP_HANDLE p;
	IDPJNI_GET_HANDLE(p, IDPCIMPL_ConfigurationSetup, "ConfigurationSetup::Destruct()")
	DestroyConfigurationSetup(p);
}

JNIEXPORT void JNICALL Java_soplex_saferpay_ConfigurationSetup_SetConfiguration
  (JNIEnv *env, jobject obj, jstring Path)
{
	IDPCIMPL_ConfigurationSetup *p = 0;

	IDPJNI_GET_AND_CHECK_HANDLE1(p, IDPCIMPL_ConfigurationSetup,
		"ConfigurationSetup::SetConfiguration()")

	const char *pszPath = env->GetStringUTFChars(Path, 0);
	try
	{
		p->SetConfiguration(mbstoSTRING(pszPath));

	}
	catch(IDP_Exception e)
	{
		const TCHAR *msg = e.ErrorString.c_str();
		char *buf = new char[tcslen(msg)+50];
#ifdef UNICODE
		sprintf(buf, "ConfigurationSetup::SetConfiguration(): %S", msg);
#else
		sprintf(buf, "ConfigurationSetup::SetConfiguration(): %s", msg);
#endif
		IDPJNI_THROW1(buf, e.hr)
		delete buf;
	}
	env->ReleaseStringUTFChars(Path, pszPath);
}

JNIEXPORT void JNICALL Java_soplex_saferpay_ConfigurationSetup_Reconfigure
  (JNIEnv *env, jobject obj, jstring ServerURL, jstring UserName, jstring Password)
{
	IDPCIMPL_ConfigurationSetup *p = 0;

	IDPJNI_GET_AND_CHECK_HANDLE1(p, IDPCIMPL_ConfigurationSetup,
		"ConfigurationSetup::Reconfigure()")

	const char *pszServerURL = env->GetStringUTFChars(ServerURL, 0);
	const char *pszUserName = env->GetStringUTFChars(UserName, 0);
	const char *pszPassword = env->GetStringUTFChars(Password, 0);
	try
	{
		p->Reconfigure(mbstoSTRING(pszServerURL), mbstoSTRING(pszUserName), mbstoSTRING(pszPassword));
	}
	catch(IDP_Exception e)
	{
		const TCHAR *msg = e.ErrorString.c_str();
		char *buf = new char[tcslen(msg)+50];
#ifdef UNICODE
		sprintf(buf, "ConfigurationSetup::Reconfigure(): %S", msg);
#else
		sprintf(buf, "ConfigurationSetup::Reconfigure(): %s", msg);
#endif
		IDPJNI_THROW1(buf, e.hr)
		delete buf;
	}
	env->ReleaseStringUTFChars(ServerURL, pszServerURL);
	env->ReleaseStringUTFChars(UserName, pszUserName);
	env->ReleaseStringUTFChars(Password, pszPassword);
}

JNIEXPORT void JNICALL Java_soplex_saferpay_ConfigurationSetup_SetOption
  (JNIEnv *env, jobject obj, jstring Name, jstring Value)
{
	IDPCIMPL_ConfigurationSetup *p = 0;

	IDPJNI_GET_AND_CHECK_HANDLE1(p, IDPCIMPL_ConfigurationSetup,
		"ConfigurationSetup::SetOption()")

	if((Name == NULL) || (Value == NULL))
	{
		IDPJNI_THROW1("ConfigurationSetup::SetOption(): Value or Name of the Attribute is NULL",
			IDP_E_UNEXPECTED)
		return;
	}

	const char *pszName = env->GetStringUTFChars(Name, 0);
	const char *pszValue = env->GetStringUTFChars(Value, 0);
	try
	{
		p->SetOption(mbstoSTRING(pszName), mbstoSTRING(pszValue));
	}
	catch(IDP_Exception e)
	{
		const TCHAR *msg = e.ErrorString.c_str();
		char *buf = new char[tcslen(msg)+50];
#ifdef UNICODE
		sprintf(buf, "ConfigurationSetup::SetOption(): %S", msg);
#else
		sprintf(buf, "ConfigurationSetup::SetOption(): %s", msg);
#endif
		IDPJNI_THROW1(buf, e.hr)
		delete buf;
	}
	env->ReleaseStringUTFChars(Name, pszName);
	env->ReleaseStringUTFChars(Value, pszValue);
}

JNIEXPORT jstring JNICALL Java_soplex_saferpay_ConfigurationSetup_GetAttribute
  (JNIEnv *env, jobject obj, jstring File, jstring Name)
{
	IDPCIMPL_ConfigurationSetup *p = 0;

	IDPJNI_GET_AND_CHECK_HANDLE2(p, IDPCIMPL_ConfigurationSetup,
		"ConfigurationSetup::GetAttribute()",
		env->NewStringUTF(""))

	const char *pszFile = env->GetStringUTFChars(File, 0);
	const char *pszName = env->GetStringUTFChars(Name, 0);

	STRING sValue(_T(""));
	try
	{
		p->GetAttribute(mbstoSTRING(pszFile), mbstoSTRING(pszName), sValue);
		env->ReleaseStringUTFChars(File, pszFile);
		env->ReleaseStringUTFChars(Name, pszName);
		return env->_NewString(sValue);
	}
	catch(IDP_Exception e)
	{
		if(e.hr != IDP_E_NOT_FOUND)
		{
			const TCHAR *msg = e.ErrorString.c_str();
			char *buf = new char[tcslen(msg)+50];
#ifdef UNICODE
			sprintf(buf, "ConfigurationSetup::GetAttribute(): %S", msg);
#else
			sprintf(buf, "ConfigurationSetup::GetAttribute(): %s", msg);
#endif
			IDPJNI_THROW2(buf, e.hr, env->NewStringUTF(""))
			delete buf;
		}
		env->ReleaseStringUTFChars(File, pszFile);
		env->ReleaseStringUTFChars(Name, pszName);
		return env->NewStringUTF("");
	}
}

JNIEXPORT void JNICALL Java_soplex_saferpay_ConfigurationSetup_SetAttribute
  (JNIEnv *env, jobject obj, jstring File, jstring Name, jstring newVal)
{
	IDPCIMPL_ConfigurationSetup *p = 0;

	IDPJNI_GET_AND_CHECK_HANDLE1(p, IDPCIMPL_ConfigurationSetup,
		"ConfigurationSetup::SetAttribute()")

	if((File == NULL) || (Name == NULL) || (newVal == NULL))
	{
		IDPJNI_THROW1("ConfigurationSetup::SetAttribute(): Value or File, Name or newVal is NULL",
			IDP_E_UNEXPECTED)
		return;
	}

	const char *pszName = env->GetStringUTFChars(Name, 0);
	const char *pszNewVal = env->GetStringUTFChars(newVal, 0);
	const char *pszFile = env->GetStringUTFChars(File, 0);
	try
	{
		p->SetAttribute(mbstoSTRING(pszFile), mbstoSTRING(pszName), mbstoSTRING(pszNewVal));
	}
	catch(IDP_Exception e)
	{
		const TCHAR *msg = e.ErrorString.c_str();
		char *buf = new char[tcslen(msg)+50];
#ifdef UNICODE
		sprintf(buf, "ConfigurationSetup::SetAttribute(): %S", msg);
#else
		sprintf(buf, "ConfigurationSetup::SetAttribute(): %s", msg);
#endif
		IDPJNI_THROW1(buf, e.hr)
		delete buf;
	}
	env->ReleaseStringUTFChars(Name, pszName);
	env->ReleaseStringUTFChars(newVal, pszNewVal);
	env->ReleaseStringUTFChars(File, pszFile);
}

}	// extern "C"

