// jMessageFactory.cpp

#include <stdio.h>

#include "../idputil.h"
#include "../idpapi.h"
#include "../cppMessageFactory.h"
#include "jMessageFactory.h"
#include "idpjni.h"

extern "C" {

JNIEXPORT void JNICALL Java_soplex_saferpay_MessageFactory_nConstruct
  (JNIEnv *env, jobject obj)
{
	jfieldID fid;
	jint Handle;
	jclass cls = env->GetObjectClass(obj);

	fid = env->GetFieldID(cls, "m_Handle", "I");
	if (fid == 0)
	{
		IDPJNI_THROW1("MessageFactory::Construct(): member m_Handle not found", IDP_E_UNEXPECTED)
		return;
	}
	Handle = env->GetIntField(obj, fid);
	if(Handle)
	{
		IDPJNI_THROW1("MessageFactory::Construct(): m_Handle != 0", IDP_E_UNEXPECTED)
		return;
	}
	env->SetIntField(obj, fid, (jint)CreateMessageFactory());
}

JNIEXPORT void JNICALL Java_soplex_saferpay_MessageFactory_nDestruct
  (JNIEnv *env, jobject obj)
{
	IDP_HANDLE p;
	IDPJNI_GET_HANDLE(p, IDPCIMPL_MessageFactory, "MessageFactory::Destruct()")
	DestroyMessageFactory(p);
}

JNIEXPORT jint JNICALL Java_soplex_saferpay_MessageFactory_nVerifyPayConfirm
  (JNIEnv *env, jobject obj, jstring Data, jstring Signature)
{
	IDPCIMPL_MessageFactory *p = 0;
	
	IDPJNI_GET_AND_CHECK_HANDLE2(p, IDPCIMPL_MessageFactory,
		"MessageFactory::VerifyPayConfirm()", 0)

	const char *pszData = env->GetStringUTFChars(Data, 0);
	const char *pszSignature = env->GetStringUTFChars(Signature, 0);

	try
	{
		jint r = (jint)p->VerifyPayConfirm(mbstoSTRING(pszData), mbstoSTRING(pszSignature));
		env->ReleaseStringUTFChars(Data, pszData);
		env->ReleaseStringUTFChars(Signature, pszSignature);
		return r;
	}
	catch(IDP_Exception e)
	{
		const TCHAR *msg = e.ErrorString.c_str();
		char *buf = new char[tcslen(msg)+50];
#ifdef UNICODE
		sprintf(buf, "MessageFactory::VerifyPayConfirm(): %S", msg);
#else
		sprintf(buf, "MessageFactory::VerifyPayConfirm(): %s", msg);
#endif
		IDPJNI_THROW2(buf, e.hr, 0)

		delete buf;
		env->ReleaseStringUTFChars(Data, pszData);
		env->ReleaseStringUTFChars(Signature, pszSignature);
		return 0;
	}
}

JNIEXPORT void JNICALL Java_soplex_saferpay_MessageFactory_Open
  (JNIEnv *env, jobject obj, jstring Path)
{
	IDPCIMPL_MessageFactory *p = 0;
	
	IDPJNI_GET_AND_CHECK_HANDLE1(p, IDPCIMPL_MessageFactory, "MessageFactory::Open()")

	const char *pszPath = env->GetStringUTFChars(Path, 0);
	try
	{
		p->Open(mbstoSTRING(pszPath));
	}
	catch(IDP_Exception e)
	{
		const TCHAR *msg = e.ErrorString.c_str();
		char *buf = new char[tcslen(msg)+50];
#ifdef UNICODE
		sprintf(buf, "MessageFactory::Open(): %S", msg);
#else
		sprintf(buf, "MessageFactory::Open(): %s", msg);
#endif
		IDPJNI_THROW1(buf, e.hr)

		delete buf;
	}
	env->ReleaseStringUTFChars(Path, pszPath);
}

JNIEXPORT jint JNICALL Java_soplex_saferpay_MessageFactory_nCreatePayInit
  (JNIEnv *env, jobject obj)
{
	IDPCIMPL_MessageFactory *p = 0;
	
	IDPJNI_GET_AND_CHECK_HANDLE2(p, IDPCIMPL_MessageFactory,
		"MessageFactory::CreatePayInit()", 0)

	try
	{
		return (jint)p->CreatePayInit();
	}
	catch(IDP_Exception e)
	{
		const TCHAR *msg = e.ErrorString.c_str();
		char *buf = new char[tcslen(msg)+50];
#ifdef UNICODE
		sprintf(buf, "MessageFactory::CreatePayInit(): %S", msg);
#else
		sprintf(buf, "MessageFactory::CreatePayInit(): %s", msg);
#endif
		IDPJNI_THROW2(buf, e.hr, 0)

		delete buf;
		return 0;
	}
}

JNIEXPORT jint JNICALL Java_soplex_saferpay_MessageFactory_nCreatePayComplete
  (JNIEnv *env, jobject obj, jstring Id, jstring Token)
{
	IDPCIMPL_MessageFactory *p = 0;

	IDPJNI_GET_AND_CHECK_HANDLE2(p, IDPCIMPL_MessageFactory,
		"MessageFactory::CreatePayComplete()", 0)

	const char *pszId = env->GetStringUTFChars(Id, 0);
	const char *pszToken = env->GetStringUTFChars(Token, 0);

	try
	{
		jint r = (jint)p->CreatePayComplete(mbstoSTRING(pszId), mbstoSTRING(pszToken));
		env->ReleaseStringUTFChars(Id, pszId);
		env->ReleaseStringUTFChars(Token, pszToken);
		return r;
	}
	catch(IDP_Exception e)
	{
		const TCHAR *msg = e.ErrorString.c_str();
		char *buf = new char[tcslen(msg)+50];
#ifdef UNICODE
		sprintf(buf, "MessageFactory::CreatePayComplete(): %S", msg);
#else
		sprintf(buf, "MessageFactory::CreatePayComplete(): %s", msg);
#endif
		IDPJNI_THROW2(buf, e.hr, 0)
		env->ReleaseStringUTFChars(Id, pszId);
		env->ReleaseStringUTFChars(Token, pszToken);
		delete buf;

		return 0;
	}
}

JNIEXPORT jint JNICALL Java_soplex_saferpay_MessageFactory_nCreateRequest
  (JNIEnv *env, jobject obj, jstring MsgType)
{
	IDPCIMPL_MessageFactory *p = 0;

	IDPJNI_GET_AND_CHECK_HANDLE2(p, IDPCIMPL_MessageFactory,
		"MessageFactory::CreateRequest()", 0)

	const char *pszMsgType = env->GetStringUTFChars(MsgType, 0);

	try
	{
		jint r = (jint)p->CreateRequest(mbstoSTRING(pszMsgType));
		env->ReleaseStringUTFChars(MsgType, pszMsgType);
		return r;
	}
	catch(IDP_Exception e)
	{
		const TCHAR *msg = e.ErrorString.c_str();
		char *buf = new char[tcslen(msg)+50];
#ifdef UNICODE
		sprintf(buf, "MessageFactory::CreateRequest(): %S", msg);
#else
		sprintf(buf, "MessageFactory::CreateRequest(): %s", msg);
#endif
		IDPJNI_THROW2(buf, e.hr, 0)
		env->ReleaseStringUTFChars(MsgType, pszMsgType);
		delete buf;

		return 0;
	}
}

}	// extern "C"



