// jMessageObject.cpp

#include <stdio.h>

#include "../idputil.h"
#include "../idpapi.h"
#include "../cppMessageObject.h"
#include "jMessageObject.h"
#include "idpjni.h"

extern "C" {

JNIEXPORT void JNICALL Java_soplex_saferpay_MessageObject_nConstruct
  (JNIEnv *env, jobject obj)
{
	jfieldID fid;
	jint Handle;
	jclass cls = env->GetObjectClass(obj);

	fid = env->GetFieldID(cls, "m_Handle", "I");
	if (fid == 0)
	{
		IDPJNI_THROW1("MessageObject::Construct(): member m_Handle not found",
			IDP_E_UNEXPECTED)
		return;
	}
	Handle = env->GetIntField(obj, fid);
	if(Handle)
	{
		IDPJNI_THROW1("MessageObject::Construct(): m_Handle != 0", IDP_E_UNEXPECTED)
		return;
	}
	env->SetIntField(obj, fid, (jint)CreateMessageObject());
}

JNIEXPORT void JNICALL Java_soplex_saferpay_MessageObject_nDestruct
  (JNIEnv *env, jobject obj)
{
	IDP_HANDLE p;
	IDPJNI_GET_HANDLE(p, IDPCIMPL_MessageObject, "MessageObject::Destruct()")
	DestroyMessageObject(p);
}

JNIEXPORT jstring JNICALL Java_soplex_saferpay_MessageObject_GetURL
  (JNIEnv *env, jobject obj)
{
	IDPCIMPL_MessageObject *p;
	
	IDPJNI_GET_AND_CHECK_HANDLE2(p, IDPCIMPL_MessageObject, "MessageObject::GetURL()",
		env->NewStringUTF(""))

	STRING str = _T("");
	try
	{
		p->GetURL(str);
		return env->_NewString(str);
	}
	catch(IDP_Exception e)
	{
		const TCHAR *msg = e.ErrorString.c_str();
		char *buf = new char[tcslen(msg)+50];
#ifdef UNICODE
		sprintf(buf, "MessageObject::GetURL(): %S", msg);
#else
		sprintf(buf, "MessageFactory::GetURL(): %s", msg);
#endif
		IDPJNI_THROW2(buf, e.hr, env->NewStringUTF(""))
		delete buf;
		return env->NewStringUTF("");
	}
}

JNIEXPORT void JNICALL Java_soplex_saferpay_MessageObject_SetAttribute
  (JNIEnv *env, jobject obj, jstring Name, jstring Value)
{

	IDPCIMPL_MessageObject *p;
	
	IDPJNI_GET_AND_CHECK_HANDLE1(p, IDPCIMPL_MessageObject, "MessageObject::SetAttribute()")

	if((Name == NULL) || (Value == NULL))
	{
		IDPJNI_THROW1("MessageObject::SetAttribute(): Value or Name of the Attribute is NULL",
			IDP_E_UNEXPECTED)
		return;
	}

	const char *pszName = env->GetStringUTFChars(Name, 0);
	const char *pszValue = env->GetStringUTFChars(Value, 0);
	try
	{
		p->SetAttribute(mbstoSTRING(pszName), mbstoSTRING(pszValue));
	}
	catch(IDP_Exception e)
	{
		const TCHAR *msg = e.ErrorString.c_str();
		char *buf = new char[tcslen(msg)+50];
#ifdef UNICODE
		sprintf(buf, "MessageObject::SetAttribute(): %S", msg);
#else
		sprintf(buf, "MessageFactory::SetAttribute(): %s", msg);
#endif
		IDPJNI_THROW1(buf, e.hr)
		delete buf;
	}
	env->ReleaseStringUTFChars(Name, pszName);
	env->ReleaseStringUTFChars(Value, pszValue);
}

JNIEXPORT jstring JNICALL Java_soplex_saferpay_MessageObject_GetAttribute
  (JNIEnv *env, jobject obj, jstring Name)
{
	IDPCIMPL_MessageObject *p;
	
	IDPJNI_GET_AND_CHECK_HANDLE2(p, IDPCIMPL_MessageObject, "MessageObject::GetAttribute()",
		env->NewStringUTF(""))

	const char *pszName = env->GetStringUTFChars(Name, 0);
	STRING Val(_T(""));
	try
	{
		p->GetAttribute(mbstoSTRING(pszName), Val);
		env->ReleaseStringUTFChars(Name, pszName);
		return env->_NewString(Val);
	}
	catch(IDP_Exception e)
	{
		if(e.hr != IDP_E_NOT_FOUND)
		{
			const TCHAR *msg = e.ErrorString.c_str();
			char *buf = new char[tcslen(msg)+50];
#ifdef UNICODE
			sprintf(buf, "MessageObject::GetAttribute(): %S", msg);
#else
			sprintf(buf, "MessageFactory::GetAttribute(): %s", msg);
#endif
			IDPJNI_THROW2(buf, e.hr, env->NewStringUTF(""))
			delete buf;
		}
		env->ReleaseStringUTFChars(Name, pszName);
		return env->NewStringUTF("");
	}
}

JNIEXPORT void JNICALL Java_soplex_saferpay_MessageObject_Capture
  (JNIEnv *env, jobject obj)
{
	IDPCIMPL_MessageObject *p;
	
	IDPJNI_GET_AND_CHECK_HANDLE1(p, IDPCIMPL_MessageObject, "MessageObject::Capture()")

	try
	{
		p->Capture();
		return;
	}
	catch(IDP_Exception e)
	{
		const TCHAR *msg = e.ErrorString.c_str();
		char *buf = new char[tcslen(msg)+50];
#ifdef UNICODE
		sprintf(buf, "MessageObject::Capture(): %S", msg);
#else
		sprintf(buf, "MessageFactory::Capture(): %s", msg);
#endif
		IDPJNI_THROW1(buf, e.hr)
		delete buf;
		return;
	}
}

JNIEXPORT jint JNICALL Java_soplex_saferpay_MessageObject_nExecute
  (JNIEnv *env, jobject obj)
{
	IDPCIMPL_MessageObject *p;
	
	IDPJNI_GET_AND_CHECK_HANDLE2(p, IDPCIMPL_MessageObject, "MessageObject::Execute()", 0)

	try
	{
		return (jint)p->Execute();
	}
	catch(IDP_Exception e)
	{
		const TCHAR *msg = e.ErrorString.c_str();
		char *buf = new char[tcslen(msg)+50];
#ifdef UNICODE
		sprintf(buf, "MessageObject::Execute(): %S", msg);
#else
		sprintf(buf, "MessageFactory::Execute(): %s", msg);
#endif
		IDPJNI_THROW2(buf, e.hr, 0)
		delete buf;
		return 0;
	}
}

JNIEXPORT void JNICALL Java_soplex_saferpay_MessageObject_Open
  (JNIEnv *env, jobject obj, jstring Path)
{
	IDPCIMPL_MessageObject *p;
	
	IDPJNI_GET_AND_CHECK_HANDLE1(p, IDPCIMPL_MessageObject, "MessageObject::Open()")

	const char *pszPath = env->GetStringUTFChars(Path, 0);
	try
	{
		p->Open(mbstoSTRING(pszPath));

	}
	catch(IDP_Exception e)
	{
		const TCHAR *msg = e.ErrorString.c_str();
		char *buf = new char[tcslen(msg)+50];
#ifdef UNICODE
		sprintf(buf, "MessageObject::Open(): %S", msg);
#else
		sprintf(buf, "MessageFactory::Open(): %s", msg);
#endif
		IDPJNI_THROW1(buf, e.hr)
		delete buf;
	}
	env->ReleaseStringUTFChars(Path, pszPath);
}

JNIEXPORT void JNICALL Java_soplex_saferpay_MessageObject_FromString
  (JNIEnv *env, jobject obj, jstring Message)
{
	IDPCIMPL_MessageObject *p;
	
	IDPJNI_GET_AND_CHECK_HANDLE1(p, IDPCIMPL_MessageObject, "MessageObject::FromString()")

	const char *pszMessage = env->GetStringUTFChars(Message, 0);
	try
	{
		p->FromString(mbstoSTRING(pszMessage));
	}
	catch(IDP_Exception e)
	{
		const TCHAR *msg = e.ErrorString.c_str();
		char *buf = new char[tcslen(msg)+50];
#ifdef UNICODE
		sprintf(buf, "MessageObject::FromString(): %S", msg);
#else
		sprintf(buf, "MessageFactory::FromString(): %s", msg);
#endif
		IDPJNI_THROW1(buf, e.hr)
		delete buf;
	}
	env->ReleaseStringUTFChars(Message, pszMessage);
}

JNIEXPORT jstring JNICALL Java_soplex_saferpay_MessageObject_ToString
  (JNIEnv *env, jobject obj)
{
	IDPCIMPL_MessageObject *p;
	
	IDPJNI_GET_AND_CHECK_HANDLE2(p, IDPCIMPL_MessageObject, "MessageObject::GetAttribute()",
		env->NewStringUTF(""))

	STRING Val(_T(""));
	try
	{
		Val = p->ToString();
		return env->_NewString(Val);
	}
	catch(IDP_Exception e)
	{
		const TCHAR *msg = e.ErrorString.c_str();
		char *buf = new char[tcslen(msg)+50];
#ifdef UNICODE
		sprintf(buf, "MessageObject::ToString(): %S", msg);
#else
		sprintf(buf, "MessageFactory::ToString(): %s", msg);
#endif
		IDPJNI_THROW2(buf, e.hr, env->NewStringUTF(""))
		delete buf;
		return env->NewStringUTF("");
	}
}

}	// extern "C"



