package soplex::saferpay::ConfigurationSetup;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

require Exporter;
require DynaLoader;
require AutoLoader;

@ISA = qw(Exporter DynaLoader);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
@EXPORT = qw();
@EXPORT_OK = qw(ConfigurationSetup);
$VERSION = '0.01';

bootstrap soplex::saferpay::ConfigurationSetup $VERSION;

# Preloaded methods go here.


sub new
{
	my $classname = shift;
	my $self = {};
	bless($self, $classname);
	$self->_init(@_);
	
	return $self;
}

sub SetConfiguration
{
	my $self = shift;
	nSetConfiguration($self->{"handle"}, @_) if($self->{"handle"}) or die "invalid handle";
	1;
}

sub Reconfigure
{
	my $self = shift;
	nReconfigure($self->{"handle"}, @_) if($self->{"handle"}) or die "invalid handle";
	1;
}

sub SetAttribute
{
	my $self = shift;
	nSetAttribute($self->{"handle"}, @_) if($self->{"handle"}) or die "invalid handle";
	1;
}

sub GetAttribute
{
	my $self = shift;
	return nGetAttribute($self->{"handle"}, @_) if($self->{"handle"}) or die "invalid handle";
}

sub _init
{
	my $self = shift;
	my $handle = shift || nConstruct();
	my %param = @_;
	$param{"handle"} = $handle;
	%{$self} = %param;
}

sub DESTROY
{
	my $self = shift;
	nDestruct($self->{"handle"}) if $self->{"handle"};
}


# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__
# Below is the stub of documentation for your module. You better edit it!

=head1 NAME

soplex::saferpay::ConfigurationSetup - Perl extension for the Saferpay Application Component.

=head1 SYNOPSIS

  use soplex::saferpay::ConfigurationSetup;
  # create a ConfigurationSetup object
  $CS = ConfigurationSetup->new;
  
  # choose the directory for your Saferpay account
  $CS->SetConfiguration("/path/to/your/saferpay/account");
  
  # configure your Saferpay account
  $CS->Reconfigure("https://www.saferpay.com/user/setup.asp", "e99867001", "XAjc3Kna");

=head1 DESCRIPTION

refer to http://www.saferpay.com/help/ for a complete description of the Saferpay Application Component.

=head1 AUTHOR

Soplex AG, support@saferpay.com

=head1 SEE ALSO

perl(1), perlxs(1).

=cut
