package soplex::saferpay::MessageFactory;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

use soplex::saferpay::MessageObject;

require Exporter;
require DynaLoader;
require AutoLoader;

@ISA = qw(Exporter DynaLoader);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
@EXPORT = qw();
@EXPORT_OK = qw(MessageFactory);
$VERSION = '0.01';

bootstrap soplex::saferpay::MessageFactory $VERSION;

# Preloaded methods go here.

sub new
{
	my $classname = shift;
	my $self = {};
	bless($self, $classname);
	$self->_init(@_);
	
	return $self;
}

sub Open
{
	my $self = shift;
	nOpen($self->{"handle"}, @_) if($self->{"handle"}) or die "invalid handle";
	1;
}

sub CreatePayInit
{
	my $self = shift;
	my $MO = nCreatePayInit($self->{"handle"}, @_) if($self->{"handle"}) or die "invalid handle";
	return soplex::saferpay::MessageObject->new($MO);
}

sub VerifyPayConfirm
{
	my $self = shift;
	my $MO = nVerifyPayConfirm($self->{"handle"}, @_) if($self->{"handle"}) or die "invalid handle";
	return soplex::saferpay::MessageObject->new($MO);
}

sub CreatePayComplete
{
	my $self = shift;
	my $MO = nCreatePayComplete($self->{"handle"}, @_) if($self->{"handle"}) or die "invalid handle";
	return soplex::saferpay::MessageObject->new($MO);
}

sub CreateRequest
{
	my $self = shift;
	my $MO = nCreateRequest($self->{"handle"}, @_) if($self->{"handle"}) or die "invalid handle";
	return soplex::saferpay::MessageObject->new($MO);
}

sub _init
{
	my $self = shift;
	my $handle = shift || nConstruct();
	my %param = @_;
	$param{"handle"} = $handle;
	%{$self} = %param;
}

sub DESTROY
{
	my $self = shift;
	nDestruct($self->{"handle"}) if $self->{"handle"};
}




# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__
# Below is the stub of documentation for your module. You better edit it!

=head1 NAME

soplex::saferpay::MessageFactory - Perl extension for the Saferpay Application Component.

=head1 SYNOPSIS

  use soplex::saferpay::MessageFactory qw(MessageFactory);
  
  # create a MessageFactory object
  $MF = MessageFactory->new();
  
  # choose the directory for your Saferpay account
  $MF->Open("/path/to/your/Saferpay/account");
  
  # create a PayInit URL
  $MO = $MF->CreatePayInit("DESCRIPTION" => "your product name",
  			   "AMOUNT" => "120",
  			   "lots" => "of",
  			   "other" => "attributes");
  # $payInit is the link to the Saferpay Virtual Terminal as a string
  $payInit = $MO->GetURL();
  
  # verify a PayConfirm message
  $MO = $MF->VerifyPayConfirm($data, $signature,
  				"additional" => "attributes");
  				
  # persist the message object to do the settlement later on
  $payConfirmString = $MO->ToSTRING();
  
  # create a MessageObject from a string
  $MO = soplex::saferpay::MessageObject->new();
  $MO->FromSTRING($payConfirmString);
     				
  # create a PayComplete message
  $MO = $MF->CreatePayComplete($MO->GetAttribute("ID"), $MO->GetAttribute("TOKEN"));
  
  # now do the settlement
  $MO->Capture();
  
  # perform a final balance
  $MO = $MF->CreatePayComplete("<unused>", "<unused>",
  			       "ACCOUNTID" => "your account ID");
  $MO->Capture();
  
  
  

=head1 DESCRIPTION

refer to http://www.saferpay.com/help/ for a complete description of the Saferpay Application Component.

=head1 AUTHOR

Soplex AG, support@saferpay.com

=head1 SEE ALSO

perl(1), perlxs(1).

=cut
