#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

#include "../../idpapi.h"
#include "../../idperrc.h"

#define CHECK(msg) { if(hr != IDP_S_OK)\
	{\
		char szErr[1024];\
		long cbSize = sizeof(szErr) - 2;\
		sprintf(szErr, ": ");\
		croak(msg " (Error: 0x%08X%s)", hr,\
		IDP_GetErrorStringFromResult(hr, &cbSize, &szErr[2]) ? szErr : ""); } }
#define CHECK_CLEAN(msg, ptr) { if(hr != IDP_S_OK) { Safefree(ptr); CHECK(msg); } }

MODULE = soplex::saferpay::MessageFactory		PACKAGE = soplex::saferpay::MessageFactory		

void *
nConstruct()
	PREINIT:
	IDP_HANDLE result = NULL;
	CODE:
	result = CreateMessageFactory();
	if(!(RETVAL = result)) croak("unable to create MessageFactory object;"
					" Server probably out of memory");
	OUTPUT:
	RETVAL

void
nDestruct(hMF)
	void *hMF;
	CODE:
	DestroyMessageFactory(hMF);
	
void
nOpen(hMF, pszPath)
	void *hMF;
	char *pszPath;
	PREINIT:
	HRESULT hr;
	CODE:
	hr = MessageFactory_Open(hMF, pszPath);
	CHECK("Open failed");
	
void *
nCreatePayInit(hMF, ...)
	void *hMF;
	PREINIT:
	HRESULT hr;
	void *hMO = NULL;
	int i;
	CODE:
	hr = MessageFactory_CreatePayInit(hMF, &hMO);
	CHECK("CreatePayInit failed");
	for(i = 1; i < items - 1; i += 2)
	{
		hr = MessageObject_SetAttribute(hMO, SvPV(ST(i), SvCUR(ST(i))), SvPV(ST(i+1), SvCUR(ST(i+1))));
		CHECK("CreatePayInit failed while calling SetAttribute");
	}
	RETVAL = hMO;
	OUTPUT:
	RETVAL
	
void *
nVerifyPayConfirm(hMF, pszData, pszSignature, ...)
	void *hMF;
	char *pszData;
	char *pszSignature;
	PREINIT:
	HRESULT hr;
	void *hMO = NULL;
	CODE:
	hr = MessageFactory_VerifyPayConfirm(hMF, pszData, pszSignature, &hMO);
	CHECK("VerifyPayConfirm failed");
	RETVAL = hMO;
	OUTPUT:
	RETVAL
	
void *
nCreatePayComplete(hMF, pszId, pszToken)
	void *hMF;
	char *pszId;
	char *pszToken;
	PREINIT:
	HRESULT hr;
	void *hMO = NULL;
	int i;
	CODE:
	hr = MessageFactory_CreatePayComplete(hMF, pszId, pszToken, &hMO);
	CHECK("CreatePayComplete failed");
	for(i = 3; i < items - 1; i += 2)
	{
		hr = MessageObject_SetAttribute(hMO, SvPV(ST(i), SvCUR(ST(i))), SvPV(ST(i+1), SvCUR(ST(i+1))));
		CHECK("CreatePayComplete failed while calling SetAttribute");
	}
	RETVAL = hMO;
	OUTPUT:
	RETVAL
	
void *
nCreateRequest(hMF, pszMsgType, ...)
	void *hMF;
	char *pszMsgType;
	PREINIT:
	HRESULT hr;
	void *hMO = NULL;
	int i;
	CODE:
	hr = MessageFactory_CreateRequest(hMF, pszMsgType, &hMO);
	CHECK("CreateRequest failed");
	for(i = 2; i < items - 1; i += 2)
	{
		hr = MessageObject_SetAttribute(hMO, SvPV(ST(i), SvCUR(ST(i))), SvPV(ST(i+1), SvCUR(ST(i+1))));
		CHECK("CreateRequest failed while calling SetAttribute");
	}
	RETVAL = hMO;
	OUTPUT:
	RETVAL
	
