# Before `make install' is performed this script should be runnable with
# `make test'. After `make install' it should work as `perl test.pl'

######################### We start with some black magic to print on failure.

# Change 1..1 below to 1..last_test_to_print .
# (It may become useful if the test is moved to ./t subdirectory.)

BEGIN
{
	$| = 1; print "1..16\n";
	push @INC, "../MessageObject/blib/lib", "../MessageObject/blib/arch";
}
END {print "not ok 1\n" unless $loaded;}
use soplex::saferpay::MessageFactory qw(MessageFactory);
use soplex::saferpay::MessageObject qw(MessageObject);
$loaded = 1;
print "ok 1\n";

######################### End of black magic.

# Insert your test code below (better if it prints "ok 13"
# (correspondingly "not ok 13") depending on the success of chunk 13
# of the test code):

my $MF = 0;

eval {
	$MF = soplex::saferpay::MessageFactory->new();
	print "ok 2\n";
} or die "not ok 2\n", $@, "\n";

print "opening the MessageFactory in ../../out...\n";

eval {
	$MF->Open("../../out");
	printf "ok 3\n";
} or die "not ok 3\n", $@, "\n";

print "creating the PayInit URL ...\n";
eval {
	$MO = $MF->CreatePayInit("DESCRIPTION" => "a " . ("very " x $i) . "expensive notebook",
		   "AMOUNT" => $i * 100000 + 300000,
		   "CURRENCY" => "CHF",
		   "ACCOUNTID" => "99867-94913159",
		   "SUCCESSLINK" => "http://www.mynotebook.com/success.html",
		   "FAILLINK" => "http://www.mynotebook.com/failed.html",
		   "BACKLINK" => "http://www.mynotebook.com/shop.html",
		   "DELIVERY" => "yes",
		   "ALLOWCOLLECT" => "yes");
		   
	print "ok 4\n";
} or die "not ok 4\n", $@, "\n";

eval {
	$MO->SetAttribute("AMOUNT", "100000");
	print "ok 5\n";
} or die "not ok 5\n";

eval {
	$payInit = $MO->GetURL;
	print "$payInit\n";
	print "ok 6\n";
} or die "not ok 6\n", $@, "\n";

if($MO->GetAttribute("AMOUNT") eq "100000")
{
	$MO->SetAttribute("DESCRIPTION", "a somewhat cheaper notebook");
	print "ok 7\n";
}
else
{
	print "not ok 7\n";
}

print "creating an authorization request...\n";
eval {
	$MO = $MF->CreateRequest("Authorization",
		"PAN" => "9451123100000004",
		"ACCOUNTID" => "99867-94913159",
		"EXP" => "1214",
		"AMOUNT" => "120",
		"CURRENCY" => "CHF",
		"AUTHFLAGS" => "0");
	print "ok 9\n";
} or die "not ok 8\n", $@, "\n";

print "getting the authorization response from the Saferpay host...\n";
eval {
	$response = $MO->Execute();
	print "ok 10\n";
} or die "not ok 10; \n", $@, "\n";

if($response->GetAttribute("RESULT") eq "0")
{
	print "response attributes:\n";
	print "ID: ", $response->GetAttribute("ID"), "\n";
	print "TOKEN: ", $response->GetAttribute("TOKEN"), "\n";
	print "AUTHCODE: ", $response->GetAttribute("AUTHCODE"), "\n";
	print "ACCOUNTID: ", $response->GetAttribute("ACCOUNTID"), "\n";
	print "PROVIDERID: ", $response->GetAttribute("PROVIDERID"), "\n";
	print "PROVIDERNAME: ", $response->GetAttribute("PROVIDERNAME"), "\n";
	print "CONTRACTNUMBER: ", $response->GetAttribute("CONTRACTNUMBER"), "\n";
	print "ok 11\n";
	
	print "\nsaving the MessageObject into file...\n";
	eval {
		$authResponse = $response->ToString();
		$fileName = $response->GetAttribute("ID");
		open TXFILE, ">$fileName";
		print TXFILE $authResponse;
		close TXFILE;
		print "ok 12\n";
	} or die "not ok 12\n", $@, "\n";
	
	print "loading the transaction from file to do the settlement..\n";
	eval {
		open TXFILE, "<$fileName";
		$authResponse = <TXFILE>;
		close TXFILE;
		unlink $fileName;
		undef $response;
		$response = soplex::saferpay::MessageObject->new();
		$response->Open("../../out");
		$response->FromString($authResponse);
		print "ok 13\n";
	} or die "not ok 13\n", $@, "\n";
	
	print "creating a PayComplete message...\n";
	eval {
		$MO = $MF->CreatePayComplete($response->GetAttribute("ID"), $response->GetAttribute("TOKEN"));
		print "ok 14\n";
	} or die "not ok 14\n", $@, "\n";
	
	print "capturing the transaction...\n";
	eval {
		$MO->Capture();
		print "ok 15\n";
	} or die "not ok 15\n", $@, "\n";
	
	print "doing a batch closure...\n";
	eval {
		$MO = $MF->CreatePayComplete("<unused>", "<unused>");
		$MO->SetAttribute("ACTION", "CloseBatch");
		$MO->SetAttribute("ACCOUNTID", "99867-94913159");
		$MO->Capture();
		print "ok 16\n";
	} or die "not ok 16\n", $@, "\n";
}
else
{
	die "authorization request failed; not ok 11\n";
}

		


