#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"
                                                 
#include "../../idpapi.h"
#include "../../idperrc.h"

#define CHECK(msg) { if(hr != IDP_S_OK)\
	{\
		char szErr[1024];\
		long cbSize = sizeof(szErr) - 2;\
		sprintf(szErr, ": ");\
		croak(msg " (Error: 0x%08X%s)", hr,\
		IDP_GetErrorStringFromResult(hr, &cbSize, &szErr[2]) ? szErr : ""); } }
#define CHECK_CLEAN(msg, ptr) { if(hr != IDP_S_OK) { Safefree(ptr); CHECK(msg); } }

MODULE = soplex::saferpay::MessageObject		PACKAGE = soplex::saferpay::MessageObject		

void *
nConstruct()
	PREINIT:
	IDP_HANDLE result = NULL;
	CODE:
	result = CreateMessageObject();
	if(!(RETVAL = result)) croak("unable to create MessageObject;"
					" Server probably out of memory");
	OUTPUT:
	RETVAL

void
nDestruct(hMO)
	void *hMO;
	CODE:
	DestroyMessageObject(hMO);
	
SV *
nGetURL(hMO)
	void *hMO;
	PREINIT:
	HRESULT hr;
	long cbSize = 1;
	char *pszURL = NULL;
	SV *pResult;
	CODE:
	MessageObject_GetURL(hMO, &cbSize, pszURL);
	New(0, pszURL, cbSize, char);
	if(!pszURL) croak("GetURL failed; Server out of memory");
	hr = MessageObject_GetURL(hMO, &cbSize, pszURL);
	CHECK_CLEAN("GetURL failed", pszURL);
	pResult = sv_newmortal();
	sv_setpvn(pResult, pszURL, cbSize - 1);
	Safefree(pszURL);	
	SvREFCNT_inc(pResult);
	RETVAL = pResult;
	OUTPUT:
	RETVAL
	
void
nSetAttribute(hMO, pszName, pszNewVal)
	void *hMO;
	char *pszName;
	char *pszNewVal;
	PREINIT:
	HRESULT hr;
	CODE:
	hr = MessageObject_SetAttribute(hMO, pszName, pszNewVal);
	CHECK("SetAttribute failed");

SV *
nGetAttribute(hMO, pszName)
	void *hMO;
	char *pszName;
	PREINIT:
	HRESULT hr;
	long cbSize = 1;
	char *pszValue = NULL;
	SV *pResult;
	CODE:
	MessageObject_GetAttribute(hMO, pszName, &cbSize, pszValue);
	New(0, pszValue, cbSize, char);
	if(!pszValue) croak("GetAttribute failed; Server out of memory");
	hr = MessageObject_GetAttribute(hMO, pszName, &cbSize, pszValue);
	CHECK_CLEAN("GetAttribute failed", pszValue);
	pResult = sv_newmortal();
	sv_setpvn(pResult, pszValue, cbSize - 1);
	Safefree(pszValue);	
	SvREFCNT_inc(pResult);
	RETVAL = pResult;
	OUTPUT:
	RETVAL

void
nCapture(hMO)
	void *hMO;
	PREINIT:
	HRESULT hr;
	CODE:
	hr = MessageObject_Capture(hMO);
	CHECK("Capture failed");

void *
nExecute(hMO)
	void *hMO;
	PREINIT:
	HRESULT hr;
	void *hMOResponse;
	CODE:
	hr = MessageObject_Execute(hMO, &hMOResponse);
	CHECK("Execute failed");
	RETVAL = hMOResponse;
	OUTPUT:
	RETVAL
	
SV *
nToString(hMO)
	void *hMO;
	PREINIT:
	HRESULT hr;
	char *pszMO = NULL;
	long cbSize = 1;
	SV *pResult;
	CODE:
	MessageObject_ToString(hMO, &cbSize, pszMO);
	New(0, pszMO, cbSize, char);
	if(!pszMO) croak("ToString failed; Server out of memory");
	hr = MessageObject_ToString(hMO, &cbSize, pszMO);
	CHECK_CLEAN("ToString failed", pszMO);
	pResult = sv_newmortal();
	sv_setpvn(pResult, pszMO, cbSize - 1);
	Safefree(pszMO);	
	SvREFCNT_inc(pResult);
	RETVAL = pResult;
	OUTPUT:
	RETVAL
	
void
nFromString(hMO, pszMO)
	void *hMO;
	char *pszMO;
	PREINIT:
	HRESULT hr;
	CODE:
	hr = MessageObject_FromString(hMO, pszMO);
	CHECK("FromString failed");
	
void
nOpen(hMO, pszPath)
	void *hMO;
	char *pszPath;
	PREINIT:
	HRESULT hr;
	CODE:
	hr = MessageObject_Open(hMO, pszPath);
	CHECK("Open failed");
	
