// spliturl.cpp

#include "idptypes.h"
#include "idphttp.h"
#include "ctype.h"

#ifdef UNICODE
#define ISTALNUM iswalnum
#else
#define ISTALNUM isalnum
#endif

int SplitUrl(const STRING & url, URL_INFO & ui)
{
	const TCHAR * p, * q;
	p = url.c_str();
	q = p;

	int s = 0;
	while(* p)
	{
		TCHAR c = * p;
		switch(s)
		{
		case 0:
			if(ISTALNUM(c)) break;
			if(c == ':') 
			{ 
				ui.Protocol = STRING(q, p - q);
				s = 1; break; 
			}
			return 0;

		case 1:
		case 2:
			if(c == '/') { s++; break; }
			return 0;

		case 3:
			{
				const TCHAR * t1, * t2;
				t1 = tcschr(p, '?');
				if(!t1) t1 = p + tcslen(p);
				t2 = tcschr(p, '@');
				if(t2 && (t2 < t1))
				{
					// has username password
					q = p;
					s = 4;
					break;
				}
				// no username/password
				q = p;
				s = 10;
				break;
			}

		case 4:
			// expect username
			if(c == ':') 
			{
				ui.UserName = STRING(q, p - q);
				s = 5; q = p + 1;
			}
			break;

		case 5:
			// password
			if(c == '@')
			{
				ui.Password = STRING(q, p - q);
				s = 10; q = p + 1;
			}
			break;


		case 10:
			// hostname
			if(c == ':')
			{
				// port
				ui.Host = STRING(q, p - q);
				s = 11; q = p + 1;
				break;
			}
			if(c == '/')
			{
				ui.Host = STRING(q, p - q);

				if(ui.Protocol == _T("http"))
					ui.Port = _T("80");
				else if(ui.Protocol == _T("https"))
					ui.Port = _T("443");

				s = 12;
				q = p;
			}
			break;

		case 11:
			// port
			if(c == '/')
			{
				ui.Port = STRING(q, p - q);
				s = 12;
				q = p;
			}
			break;

		case 12:
			// uri
			p += tcslen(p);
			ui.Uri = STRING(q, p - q);
			p--;
			break;
		}	

		++p;
	}

	return -1;
}
