// sxcontainer.h: SXFC container base class

#ifndef __SXCONTAINER_H__
#define __SXCONTAINER_H__

#include "sxsort.h"

class SxContainer : public SxObject
{
public:
	SxContainer() { m_capacity = 0; m_size = 0; }
	SxContainer(SX_SIZE_T initialCapacity) { m_size = 0; m_capacity = initialCapacity; }

	// SxObject
	ALLOC_IMPL(SxContainer);
	virtual SxObject& assign(SxObject& obj)
	{
		m_size = ((SxContainer&)obj).size();
		m_capacity = ((SxContainer&)obj).capacity();

		return (SxObject&)(*this);
	}
	virtual bool isInstanceOf(const TCHAR *pszClass) const
	{
		if(!TCSCMP(pszClass, _T("SxContainer")))
			return true;

		return SxObject::isInstanceOf(pszClass);
	}

	SX_SIZE_T size() { return m_size; }
	SX_SIZE_T capacity() { return m_capacity; }

protected:
	virtual void clear() { m_size = 0; m_capacity = 0; }

	SX_SIZE_T m_size;
	SX_SIZE_T m_capacity;
};

class SxFindableContainer : public SxFindable
{
public:
	SxFindableContainer() { m_capacity = 0; m_size = 0; }
	SxFindableContainer(SX_SIZE_T initialCapacity) { m_size = 0; m_capacity = initialCapacity; }

	// SxObject
	ALLOC_IMPL(SxFindableContainer);
	virtual SxObject& assign(SxObject& obj)
	{
		m_size = ((SxFindableContainer&)obj).size();
		m_capacity = ((SxFindableContainer&)obj).capacity();

		return (SxObject&)(*this);
	}
	virtual bool isInstanceOf(const TCHAR *pszClass) const
	{
		if(!TCSCMP(pszClass, _T("SxFindableContainer")))
			return true;

		return SxFindable::isInstanceOf(pszClass);
	}

	// SxFindable
	virtual bool operator <(const SxFindable&) const { return false; }
	virtual bool operator ==(const SxFindable&) const { return false; }

	SX_SIZE_T size() { return m_size; }
	SX_SIZE_T capacity() { return m_capacity; }

protected:
	virtual void clear() { m_size = 0; m_capacity = 0; }

	SX_SIZE_T m_size;
	SX_SIZE_T m_capacity;
};

#endif // __SXCONTAINER_H__
