// sxobject.h: basic object allocation/assignment interface

#ifndef __SXOBJECT_H__
#define __SXOBJECT_H__

#include "sxfctype.h"
#include "sxtchar.h"
#include "sxexcept.h"
#include "sxfcerr.h"


// debug
#ifdef WIN32
#include "crtdbg.h"
#else
#define _ASSERT(exp)
#endif


// allocator macro
#define ALLOC_IMPL(clazz) virtual Allocator getAllocator() const { return allocate; }; static void allocate(SxObject **pp) { if(!!*pp) delete *pp; *pp = new clazz; if(!*pp) throw SXFC_Exception(E_SXFC_OUT_OF_MEMORY); }

class SxObject;


typedef void (*Allocator)(SxObject **ppObj);

class SxRefCounter
{
private:
	int m_nRef;

public:
	static int m_nTotalRef;

public:
	SxRefCounter() { m_nRef = 0; m_nTotalRef ++; }
	virtual ~SxRefCounter() { m_nTotalRef--; }
	int AddRef() { return ++m_nRef; }
	int Release() {
		m_nRef--;
		int r = m_nRef;
		if(m_nRef == 0)
			delete this;
		return r;
	}
};

class SxObject : public SxRefCounter
{
public:
	SxObject() : SxRefCounter() {;};

	ALLOC_IMPL(SxObject);

	virtual SxObject& assign(const SxObject&) { return *this;}
	virtual SxObject& operator =(const SxObject& obj) { return assign(obj); }
	virtual bool isInstanceOf(const TCHAR *pszClass) const
	{
		return !TCSCMP(pszClass, _T("SxObject")) ? true : false;
	}
};

#endif // __SXOBJECT_H__
