// sxpair.h: SXFC key/value pair class

#ifndef __SXPAIR_H__
#define __SXPAIR_H__

#include "sxsort.h"
#include "sxexcept.h"
#include "sxfcerr.h"
#include "sxptr.h"
#include "sxstring.h"


class SxKeyValue : public SxFindable
{
public:
	SxKeyValue()
	{
		m_pKey = 0;
		m_pValue = 0;
	}

	SxKeyValue(const SxFindable& key, const SxObject& value) // throws SXFC_Exception
	{
		m_pKey = 0;
		m_pValue = 0;

		Allocator pfnAllocate = key.getAllocator();
		pfnAllocate((SxObject**)(&m_pKey));
		m_pKey->assign(key);

		pfnAllocate = value.getAllocator();
		pfnAllocate(&m_pValue);
		m_pValue->assign(value);
	}

	~SxKeyValue()
	{
		if(!!m_pKey)
			delete m_pKey;
		
		if(!!m_pValue)
			delete m_pValue;
	}

	// SxObject
	ALLOC_IMPL(SxKeyValue);
	virtual SxObject& assign(const SxObject&); // throws SXFC_Exception
	virtual bool isInstanceOf(const TCHAR *pszClass) const
	{
		if(!TCSCMP(pszClass, _T("SxKeyValue")))
			return true;

		return SxFindable::isInstanceOf(pszClass);
	}

	// SxFindable
	virtual bool operator <(const SxFindable&) const;
	virtual bool operator ==(const SxFindable&) const;


	// data accessors
	SxFindable& first()
	{
		return *m_pKey;
	}

	SxObject& second()
	{
		return *m_pValue;
	}
	const SxFindable& first() const
	{
		return *m_pKey;
	}

	const SxObject& second() const
	{
		return *m_pValue;
	}

private:
	SxFindable *m_pKey;
	SxObject *m_pValue;
};

class SxStrPtr : public SxFindable
{
public:
	SxStrPtr() {;}
	SxStrPtr(const SxString& key, const SxPtr& p) // throws SXFC_Exception
	{
		m_key = key;
		m_pObj = p;
	}

	// SxObject
	ALLOC_IMPL(SxStrPtr);
	virtual SxObject& assign(const SxObject& obj) // throws SXFC_Exception
	{
		m_key = (SxString&)((SxStrPtr&)obj).first();

		m_pObj = ((SxStrPtr&)obj).second();

		return *this;
	}
	virtual bool isInstanceOf(const TCHAR *pszClass) const
	{
		if(!TCSCMP(pszClass, _T("SxStrPtr")))
			return true;

		return SxFindable::isInstanceOf(pszClass);
	}

	// SxFindable
	bool operator <(const SxFindable& obj) const
	{
		if(obj.isInstanceOf(_T("SxStrPtr")))
			return (m_key < ((SxStrPtr&)obj).first());
		else
			return (m_key < obj);
	}

	bool operator ==(const SxFindable& obj) const
	{
		if(obj.isInstanceOf(_T("SxStrPtr")))
			return (m_key == ((SxStrPtr&)obj).first());
		else
			return (m_key == obj);
	}


	SxStrPtr& assign(SxPtr& p)
	{
		m_pObj = p;
		return *this;
	}

	SxStrPtr& operator =(SxPtr& p)
	{
		return assign(p);
	}


	// data accessors
	SxString& first()
	{
		return m_key;
	}

	SxPtr& second()
	{
		return m_pObj;
	}
	const SxString& first() const
	{
		return m_key;
	}

	const SxPtr& second() const
	{
		return m_pObj;
	}

private:
	SxString m_key;
	SxPtr m_pObj;
};

#endif // __SXPAIR_H__
