// sxptr.h

#ifndef __XMLPOINTER_H__
#define __XMLPOINTER_H__

#include "sxobject.h"
#include "sxexcept.h"
#include "sxfcerr.h"

class SxPtr : public SxObject
{
protected:
	SxObject *p;

public:
	SxPtr() { p = 0; }
	SxPtr(SxObject * _p) { p = _p; if(p) p->AddRef(); }
	SxPtr(const SxPtr& a) { p = a.p; if(p) p->AddRef(); }

	virtual ~SxPtr() { if(p) p->Release(); }

	// SxObject
	ALLOC_IMPL(SxPtr);

	virtual bool isInstanceOf(const TCHAR *pszClass) const
	{
		if(!TCSCMP(pszClass, _T("SxPtr")))
			return true;

		return SxObject::isInstanceOf(pszClass);
	}

	void operator = (SxObject * _p) { if(p) p->Release(); p = _p; if(p) p->AddRef(); }
	void operator = (const SxPtr& eop) { if(p) p->Release(); p = eop.p; if(p) p->AddRef(); }
	inline operator SxObject * () { return p; }
	inline bool operator!() { return p ? false : true; }
	inline const SxObject *ptr() const { return p; }
	inline SxObject *ptr() { return p; }

	// pointer comparison
	//bool operator == (const T * _p) const { return (p == _p); }

	// the invocation operator
	inline SxObject * operator->() { _ASSERT(p != NULL); return p; }
	inline const SxObject * operator->() const { _ASSERT(p != NULL); return p; }

	// the & operator is not used because of conflicts with STL
	inline SxObject ** Ptr() { _ASSERT(p == NULL); return & p; }

	// return without calling release
	inline SxObject * Detach() { SxObject * t = p; p = 0; return t; }
};

#endif

