// sxstring.cpp: sxfc string class implementation


#include "sxexcept.h"
#include "sxfcerr.h"
#include "sxstring.h"



//////////////////////////// implementation of class SxString

const SxString::size_type SxString::npos = (SxString::size_type)-1;



// public methods
SxString& SxString::append(const SxString& str) // throws SXFC_Exception
{
	if(str.c_str() && str.size())
	{
		this->grow(str.size());
		TCSCPY(&m_p[m_size], str.c_str());
		m_size += str.size();
	}

	return *this;
}

SxString& SxString::append(const TCHAR *psz) // throws SXFC_Exception
{
	if(psz)
	{
		SX_SIZE_T len = TCSLEN(psz);
		this->grow(len);
		TCSCPY(&m_p[m_size], psz);
		m_size += len;
	}

	return *this;
}

// SxObject
SxObject& SxString::assign(const SxObject& str) // throws SXFC_Exception
{
	if(&str != this)
	{
		this->clear();

		if(((const SxString&)str).c_str())
		{
			if(((const SxString&)str).size())
			{
				this->grow(((const SxString&)str).size());
				TCSCPY(m_p, ((const SxString&)str).c_str());
				m_size = ((const SxString&)str).size();
			}
			else
			{
				assign(((const SxString&)str).c_str(), 0);
			}
		}
	}

	return *this;
}

SxString& SxString::assign(const TCHAR *psz) // throws SXFC_Exception
{
	this->clear();

	if(psz)
		assign(psz, TCSLEN(psz));

	return *this;
}

SxString& SxString::assign(const TCHAR *psz, size_type len) // throws SXFC_Exception
{
	this->clear();

	if(psz)
	{
		this->grow(len);
		TCSNCPY(m_p, psz, len);
		m_p[len] = 0;
		m_size = len;
	}

	return *this;
}

int SxString::compare(const SxString& str) const
{
	int diff = (int)(m_size - str.size());

	return diff ? diff : TCSCMP(m_p, str.c_str());
}

int SxString::compare(const TCHAR *psz) const
{
	if(!psz)
	{
		return m_size;
	}
	if(!m_p)
	{
		return (0 - TCSLEN(psz));
	}
	return TCSCMP(m_p, psz);
}

// private helpers

SxString& SxString::append(const TCHAR c) // throws SXFC_Exception
{
	this->grow(1);
	m_p[m_size] = c;
	m_size++;
	m_p[m_size] = (TCHAR)0;

	return *this;
}

void SxString::clear()
{
	m_size = 0;
	m_capacity = 0;

	if(m_p)
	{
		delete m_p;
		m_p = 0;
	}
}

void SxString::grow(SxString::size_type len) // throws SXFC_Exception
{
	if(m_capacity < len + m_size + 1)
	{
		TCHAR *p;

		m_capacity = m_size + len + 1 + SXSTRING_RESERVE_CAPACITY;

		if(!(p = new TCHAR[m_capacity]))
		{
			throw SXFC_Exception(E_SXFC_OUT_OF_MEMORY);
			return;
		}

		if(m_p)
		{
			TCSCPY(p, m_p);
			delete m_p;
		}

		m_p = p;
	}
}

SxString operator +(const SxString& lhs, const SxString& rhs)
{
	SxString ret = lhs;
	ret.append(rhs);
	return ret;
}

SxString operator +(const TCHAR* psz, const SxString& str)
{
	SxString ret;

	if(psz)
		ret = psz;

	ret += str;
	return ret;
}




