// sxvector.cpp: vector class implementation for project SXFC

#include "sxexcept.h"
#include "sxfcerr.h"
#include "sxvector.h"

// minimum capacity, which an SxVector grows
#define SXVECTOR_RESERVE_CAPACITY 16L

SxVector::SxVector() : SxContainer()
{
	m_ppObj = 0;
}

SxFindableVector::SxFindableVector() : SxFindableContainer()
{
	m_ppObj = 0;
}

SxVector::SxVector(SX_SIZE_T initialCapacity)
	: SxContainer(initialCapacity) // throws SXFC_Exception
{
	m_ppObj = 0;

	if(!(m_ppObj = (const SxObject**)new SxObject*[m_capacity]))
	{
		throw SXFC_Exception(E_SXFC_OUT_OF_MEMORY);
		return;
	}

	for(SX_SIZE_T i = 0; i < m_capacity; ++i)
	{
		m_ppObj[i] = 0;
	}
}

SxFindableVector::SxFindableVector(SX_SIZE_T initialCapacity)
	: SxFindableContainer(initialCapacity) // throws SXFC_Exception
{
	m_ppObj = 0;

	if(!(m_ppObj = (const SxFindable**)new SxFindable*[m_capacity]))
	{
		throw SXFC_Exception(E_SXFC_OUT_OF_MEMORY);
		return;
	}

	for(SX_SIZE_T i = 0; i < m_capacity; ++i)
	{
		m_ppObj[i] = 0;
	}
}

// SxObject
SxObject& SxVector::assign(const SxObject& vector) // throws SXFC_Exception
{
	this->clear();

	m_capacity = ((SxVector&)vector).capacity();

	if(!(m_ppObj = (const SxObject**)new SxObject*[m_capacity]))
	{
		throw SXFC_Exception(E_SXFC_OUT_OF_MEMORY);
		return *this;
	}

	while(m_size < ((SxVector&)vector).size())
	{
		m_ppObj[m_size] = ((SxVector&)vector).at(m_size);
		m_size++;
	}

	return (SxObject&)*this;
}

SxObject& SxFindableVector::assign(const SxObject& vector) // throws SXFC_Exception
{
	this->clear();

	m_capacity = ((SxFindableVector&)vector).capacity();

	if(!(m_ppObj = (const SxFindable**)new SxFindable*[m_capacity]))
	{
		throw SXFC_Exception(E_SXFC_OUT_OF_MEMORY);
		return *this;
	}

	while(m_size < ((SxFindableVector&)vector).size())
	{
		m_ppObj[m_size] = ((SxFindableVector&)vector).at(m_size);
		m_size++;
	}

	return (SxObject&)*this;
}


void SxVector::push_back(const SxObject* pObj) //throws SXFC_Exception
{
	this->grow(m_size + 1);

	m_ppObj[m_size] = pObj;

	m_size++;
}

void SxFindableVector::push_back(const SxFindable* pObj) //throws SXFC_Exception
{
	this->grow(m_size + 1);

	m_ppObj[m_size] = pObj;

	m_size++;
}

void SxVector::pop_back()
{
	m_size--;
}

void SxFindableVector::pop_back()
{
	m_size--;
}

SxObject* SxVector::at(SX_SIZE_T position) const // throw SXFC_Exception
{
	if(m_size <= position)
		throw SXFC_Exception(E_SXFC_OUT_OF_BOUND);

	return (SxObject*)m_ppObj[position];
}

SxFindable* SxFindableVector::at(SX_SIZE_T position) const // throw SXFC_Exception
{
	if(m_size <= position)
		throw SXFC_Exception(E_SXFC_OUT_OF_BOUND);

	return (SxFindable*)m_ppObj[position];
}

void SxVector::remove(SX_SIZE_T nIndex) // throws SXFC_Exception
{
	if(nIndex >= m_size)
		throw SXFC_Exception(E_SXFC_OUT_OF_BOUND);

	m_size--;

	while(nIndex < m_size)
	{
		m_ppObj[nIndex] = m_ppObj[nIndex + 1];
		nIndex++;
	}
}

void SxFindableVector::remove(SX_SIZE_T nIndex) // throws SXFC_Exception
{
	if(nIndex >= m_size)
		throw SXFC_Exception(E_SXFC_OUT_OF_BOUND);

	m_size--;

	while(nIndex < m_size)
	{
		m_ppObj[nIndex] = m_ppObj[nIndex + 1];
		nIndex++;
	}
}

// SxFindable
bool SxFindableVector::operator <(const SxFindable& obj) const
{
	if(obj.isInstanceOf(_T("SxFindableVector")))
	{
		if(!m_size)
			return ((SxFindableVector&)obj).size() ? true : false;

		if(((SxFindableVector&)obj).size() < m_size)
			return false;

		return (*m_ppObj[0] < *((SxFindableVector&)obj).at(0));
	}
	else
	{
		if(!m_size)
			return true;

		return (*m_ppObj[0] < obj);
	}
}

bool SxFindableVector::operator ==(const SxFindable& obj) const
{
	if(obj.isInstanceOf(_T("SxFindableVector")))
	{
		if(!m_size)
			return ((SxFindableVector&)obj).size() ? true : false;

		if(m_size != ((SxFindableVector&)obj).size())
			return false;

		return (*m_ppObj[0] == *((SxFindableVector&)obj).at(0));
	}
	else
	{
		if(!m_size)
			return false;

		return (*m_ppObj[0] == obj);
	}
}


// private

// SxContainer
void SxVector::clear()
{
	if(m_ppObj)
	{
		delete m_ppObj;
		m_ppObj = 0;
	}

	SxContainer::clear();
}

void SxFindableVector::clear()
{
	if(m_ppObj)
	{
		delete m_ppObj;
		m_ppObj = 0;
	}

	SxFindableContainer::clear();
}


void SxVector::grow(SX_SIZE_T capacity)
{
	if(m_capacity >= capacity)
		return;

	capacity += SXVECTOR_RESERVE_CAPACITY;
	m_capacity = capacity;

	SxObject **ppObj = new SxObject*[capacity];
	if(!ppObj)
	{
		throw SXFC_Exception(E_SXFC_OUT_OF_MEMORY);
		return;
	}

	SX_SIZE_T i;
	for(i = 0; i < capacity; ++i)
	{
		ppObj[i] = 0;
	}

	for(i = 0; i < m_size; ++i)
	{
		ppObj[i] = (SxObject*)m_ppObj[i];
	}

	if(!!m_ppObj)
		delete m_ppObj;

	m_ppObj = (const SxObject**)ppObj;
}

void SxFindableVector::grow(SX_SIZE_T capacity)
{
	if(m_capacity >= capacity)
		return;

	capacity += SXVECTOR_RESERVE_CAPACITY;
	m_capacity = capacity;

	SxFindable **ppObj = new SxFindable*[capacity];
	if(!ppObj)
	{
		throw SXFC_Exception(E_SXFC_OUT_OF_MEMORY);
		return;
	}

	SX_SIZE_T i;
	for(i = 0; i < capacity; ++i)
	{
		ppObj[i] = 0;
	}

	for(i = 0; i < m_size; ++i)
	{
		ppObj[i] = (SxFindable*)m_ppObj[i];
	}

	if(!!m_ppObj)
		delete m_ppObj;

	m_ppObj = (const SxFindable**)ppObj;
}
