// sxvector.h: vector class definition for project SXFC

#ifndef __SXVECTOR_H__
#define __SXVECTOR_H__

#include "sxcontainer.h"


class SxVector : public SxContainer
{
public:
	typedef SxObject * iterator;

	SxVector();
	SxVector(SX_SIZE_T initialCapacity); // throws SXFC_Exception

	virtual ~SxVector()
	{
		this->clear();
	}

	// SxObject
	ALLOC_IMPL(SxVector);
	virtual SxObject& assign(const SxObject&); // throws SXFC_Exception
	virtual bool isInstanceOf(const TCHAR *pszClass) const
	{
		if(!TCSCMP(pszClass, _T("SxVector")))
			return true;

		return SxContainer::isInstanceOf(pszClass);
	}

	void push_back(const SxObject*); // throws SXFC_Exception
	void pop_back();
	SxObject *at(SX_SIZE_T) const; // throws SXFC_Exception
	void remove(SX_SIZE_T);

	SxObject *begin()
	{
		if(m_size)
			return (SxObject*)m_ppObj[0];
		else
			return 0;
	}
	SxObject *end()
	{
		return 0;
	}

private:
	// SxContainer
	virtual void clear();

	void grow(SX_SIZE_T capacity);

	const SxObject **m_ppObj;
};

class SxFindableVector : public SxFindableContainer
{
public:
	typedef SxFindable * iterator;

	SxFindableVector();
	SxFindableVector(SX_SIZE_T initialCapacity); // throws SXFC_Exception

	virtual ~SxFindableVector()
	{
		this->clear();
	}

	// SxObject
	ALLOC_IMPL(SxFindableVector);
	virtual SxObject& assign(const SxObject&); // throws SXFC_Exception
	virtual bool isInstanceOf(const TCHAR *pszClass) const
	{
		if(!TCSCMP(pszClass, _T("SxFindableVector")))
			return true;

		return SxFindableContainer::isInstanceOf(pszClass);
	}

	// SxFindable
	virtual bool operator <(const SxFindable&) const;
	virtual bool operator ==(const SxFindable&) const;

	void push_back(const SxFindable*); // throws SXFC_Exception
	void pop_back();
	SxFindable *at(SX_SIZE_T) const; // throws SXFC_Exception
	void remove(SX_SIZE_T);

	SxFindable *begin()
	{
		if(m_size)
			return (SxFindable*)m_ppObj[0];
		else
			return 0;
	}
	SxFindable *end()
	{
		return 0;
	}

private:
	// SxContainer
	virtual void clear();

	void grow(SX_SIZE_T capacity);

	const SxFindable **m_ppObj;
};

#endif // __SXVECTOR_H__
