// xmlElement.h

#ifndef __XMLELEMENT_H__
#define __XMLELEMENT_H__

class xmlElement;
class xmlElementPtr;
class xmlElementEnum;
class xmlParser;

#include "xmlcore.h"

// xml Encoding
void XMLEncode(const SxString& strIn, SxString& strOut);
void XMLDecode(const SxString& strIn, SxString& strOut);


class xmlElement : public SxFindable
{
	friend class xmlElementEnum;
	friend class xmlParser;
//private:
public:
	SxString m_tag;

	typedef xmlElementPtr element_ptr;
	typedef SxStrPtrMultimap element_map;
	typedef SxStrMap attribute_map;

	attribute_map m_attributes;
	element_map m_children; 
	xmlElement * m_parent;
	SxString m_strText;

public:
	xmlElement() : SxFindable()
	{
		m_parent = 0;
	}; 

	// SxObject
	ALLOC_IMPL(xmlElement);
	virtual bool isInstanceOf(const TCHAR *pszClass) const
	{
		if(!TCSCMP(pszClass, _T("xmlElement")))
			return true;

		return SxFindable::isInstanceOf(pszClass);
	}

	// SxFindable
	virtual bool operator <(const SxFindable& obj) const
	{
		return (m_tag < obj);
	}
	virtual bool operator ==(const SxFindable& obj) const
	{
		return (m_tag == obj);
	}

	bool getAttribute(const SxString & strName, SxString & strValue);
	bool setAttribute(const SxString & strName, const SxString & strValue);
	bool getParent(xmlElement ** ppParentElement);
	bool getChild(const SxString & strTag, long nIndex, xmlElement ** ppElement);
	void insertChild(xmlElement * pElement);
	void enumChildren(const SxString & strTag, xmlElementEnum ** ppEnum);
	bool removeChild(const SxString & strTag, long nIndex);
	void getText(SxString & strText) const;
	void setText(const SxString & strText);
	bool toStream(OSTREAM & stmOut);
	bool getTag(SxString & strTag);
};

// SxPtr<xmlElement>
class xmlElementPtr : public SxPtr
{
public:
	xmlElementPtr() { p = 0; }
	xmlElementPtr(xmlElement * _p) { p = _p; if(p) p->AddRef(); }
	xmlElementPtr(const xmlElementPtr& a) { p = a.p; if(p) p->AddRef(); }

	// SxObject
	ALLOC_IMPL(xmlElementPtr);
	virtual bool isInstanceOf(const TCHAR *pszClass) const
	{
		if(!TCSCMP(pszClass, _T("xmlElementPtr")))
			return true;

		return SxPtr::isInstanceOf(pszClass);
	}

	void operator = (SxObject *_p) { if(p) p->Release(); p = _p; if(p) p->AddRef(); }
	void operator = (const xmlElementPtr& eop)
	{
		if(p) p->Release();
		p = eop.p;
		if(p) p->AddRef();
	}
	inline operator xmlElement * () { return (xmlElement*)p; }

	// pointer comparison
	//bool operator == (const T * _p) const { return (p == _p); }

	// the invocation operator
	inline xmlElement * operator->() { _ASSERT(p != NULL); return (xmlElement*)p; }
	inline const xmlElement * operator->() const { _ASSERT(p != NULL); return (xmlElement*)p; }

	// the & operator is not used because of conflicts with STL
	inline xmlElement ** Ptr() { _ASSERT(p == NULL); return (xmlElement**)&p; }

	// return without calling release
	inline xmlElement * Detach() { xmlElement * t = (xmlElement*)p; p = 0; return t; }
};

int xmlQuerySubElement(xmlElement * pElement, SxString & strId, xmlElement ** ppElement);
bool xmlGetAttributeEx(xmlElement * pElement, const SxString & strId, SxString & strValue);
bool xmlSetAttributeEx(xmlElement * pElement, const SxString & strId, const SxString & strValue);


#endif
