// xmlParser.cpp

#include "xmlcore.h"
#include "xmlerror.h"
#include "xmlParser.h"
#include "xmltab.h"

void xmlParser::createElement(const SxString & tag, xmlElement ** ppElement)
{
	element_ptr pElement = new xmlElement;
	pElement->m_tag = tag;

	* ppElement = pElement.Detach();
	return;
}

int xmlParser::parseText(ISTREAM & in, xmlElement ** ppRoot)
{
	element_ptr pRoot = new xmlElement;
	element_ptr pCurrent = pRoot;

	SxString name, attr, strDecoded;
	int s = 1;
	TCHAR c;
	int line = 1;

	while(in.get(c).good())
	{
		const tabEntry * t = getTabEntry(xmlTable, 0, s, c);
		if(!t) 
		{ 
			return XMLE_INVALID_CHAR; 
		}

		//COUT << "line=" << line << ", s=" << t->state << ", c=" << char(c) << ", n=" << t->next << ENDL;
				
		switch(t->action)
		{
		case a_append:
			name += c;
			break;
		
		case a_create_tag:
			{
			element_ptr pTmp;
			createElement(name, pTmp.Ptr());
			pCurrent->insertChild(pTmp);
			pCurrent = pTmp;
			}
			break;

		case a_close_tag:
			{
			if(!pCurrent->m_parent)
			{
				return XMLE_INVALID_STRUCTURE;
			}
			pCurrent = pCurrent->m_parent;
			}
			break;

		case a_create_attr:
			attr = name;
			break;

		case a_attr_value:
			strDecoded = _T("");
			XMLDecode(name, strDecoded);
			pCurrent->setAttribute(attr, strDecoded);
			break;


		case a_text:
			if(!name.empty())
				pCurrent->setText(name);
			break;

		}

		// clear name entry whenever it was 
		if((t->action != a_append) && (t->action != a_keep))
		{
			name = _T("");
		}

		// not every transition removes current c
		if(!t->advance)
		{
			in.putback(c);
		}
		else
		{
			if(c == '\n')
			{
				++line;
			}
		}
		s = t->next;
	}

	if(s == 1)
	{
		// all tags must be closed
		if(!!(pCurrent->m_parent)) return XMLE_FILE_INCOMPLETE;
	}
	else
	{
		return XMLE_FILE_INCOMPLETE;
	}

	// empty file is invalid too
	if(!pRoot) return XMLE_FILE_INCOMPLETE;

	// inherit reference to returned pointer
	* ppRoot = pRoot.Detach();

	return XMLE_SUCCESS;
}
