// xmltab.cpp


#include "xmltab.h"
#include "xmlerror.h"

tabEntry xmlTable[] =
{
	// keep the list ordered by (s1, symbol). this make it possible
	// to search the list binary.

	// state 1 is text mode
	{	1,	s_tag_open,		1,	2,	a_text	},
	{	1,	~ s_tag_open,	1,	1,	a_append },

	// begin of element
	{	2,	s_space,		1,	2,	0	},
	{	2,	s_alnum,		1,	3,	a_append },
	{	2,	s_tag_end,		1,	10,	a_close_tag },

	// begin tag name (len(token) > 0) 
	{	3,	s_space,		1,	4,	a_create_tag },	// trailing space 
	{	3,	s_alnum,		1,	3,	a_append },
	{	3,	s_tag_end,		0,	18,	a_create_tag },	// must run a_close_tag right after this
	{	3,	s_tag_close,	1,	1,	a_create_tag },	

	// end tag name
	{	4,	s_space,		1,	4,	0	},		
	{	4,	s_alnum,		1,	5,	a_append },		// attribute follows
	{	4,	s_tag_end,		1,	10,	a_close_tag	},	// end tag right after token
	{	4,	s_tag_close,	1,	1,	0	},			// one level deeper now

	// begin attribute name
	{	5,	s_space,		1,	6,	a_create_attr },// don't know what follows
	{	5,	s_alnum,		1,	5,	a_append },		
	{	5,	s_equal,		1,	7,	a_create_attr },// value follows
	{	5,	s_tag_end,		0,	20,	a_create_attr },// set value and go for end
	{	5,	s_tag_close,	0,	25,	a_create_attr },

	// end attribute name
	{	6,	s_space,		1,	6,	0	},			
	{	6,	s_alnum,		0,	19,	a_attr_value },		// attribute follows, set value before
	{	6,	s_equal,		1,	7,	0	},			// value follows
	{	6,	s_tag_end,		0,	18,	a_attr_value },
	{	6,	s_tag_close,	1,	1,	a_attr_value },

	// quoted or alnum string
	{	7,	s_space,		1,	7,	0,	},
	{	7,	s_squote,		1,	8,	0,	},
	{	7,	s_dquote,		1,	9,	0,	},
	{	7,	s_alnum,		1,	30,	a_append },

	// single quotes
	{	8,	s_squote,		1,	4,	a_attr_value },
	{	8,	~ s_squote,		1,	8,	a_append },

	// double quotes
	{	9,	s_dquote,		1,	4,	a_attr_value },
	{	9,	~ s_dquote,		1,	9,	a_append },

	// end of element (with optional tag, which is ignored)
	{	10,	s_space,		1,	10,	0	},
	{	10,	s_alnum,		1,	14,	0	},
	{	10,	s_tag_close,	1,	1,	0	},

	{	14,	s_space,		1,	15,	0	},
	{	14,	s_alnum,		1,	14,	0	},
	{	14,	s_tag_close,	1,	1,	0	},

	{	15,	s_space,		1,	15,	0	},
	{	15,	s_tag_close,	1,	1,	0	},

	// / right after tag
	{	18,	s_tag_end,		1,	10,	a_close_tag	},

	// 
	{	19,	s_alnum,		1,	5,	a_append },		// attribute follows, set value before

	//
	{	20,	s_tag_end,		0,	18,	a_attr_value },// set value and go for end

	//
	{	25,	s_tag_close,	1,	1,	a_attr_value },

	{	30,	s_alnum,		1,	30,	a_append },
	{	30,	~ s_alnum,		0,	4,	a_attr_value },

	{	0 ,	0,				0,	0,	a_append	}
};

int isSpace(TCHAR c)
{
	switch(c)
	{
	case ' ':
	case '\r':
	case '\n':
	case '\t':
		return 1;

	default:
		return 0;
	}
}

int isAlnum(TCHAR c)
{
	if(('A' <= c) && (c <= 'Z')) return 1;
	if(('a' <= c) && (c <= 'z')) return 1;
	if(('0' <= c) && (c <= '9')) return 1;
	switch(c)
	{
	case '_':
		return 1;
	case '-':
		return 1;
	case '.':
		return 1;
	}
	return 0;
}

int getSymbol(TCHAR c)
{
	if(c == '<') return s_tag_open;
	if(c == '=') return s_equal;
	if(c == '/') return s_tag_end;
	if(c == '>') return s_tag_close;
	if(c == '"') return s_dquote;
	if(c == '\'') return s_squote;
	if(isSpace(c)) return s_space;
	if(isAlnum(c)) return s_alnum;
	return s_other;
}

const tabEntry * getTabEntry(const tabEntry * table, int N, int s, TCHAR c)
{
	int symbol;
	const tabEntry * p = table;

	symbol = getSymbol(c);

	while(p->state)
	{
		if((p->state == s) && (p->symbol & symbol))
		{
			return p;
		}
		p++;
	}

	return 0;
}

